# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.import_search_and_replace_for_string_property import ImportSearchAndReplaceForStringProperty
from openapi_client.models.campaign.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from typing_extensions import Self

class GoogleImportOption(BaseModel):
    """
    GoogleImportOption
    """ # noqa: E501
    new_entities: Optional[StrictBool] = Field(default=None, alias="NewEntities")
    new_paused_campaigns_and_child_entities: Optional[StrictBool] = Field(default=None, alias="NewPausedCampaignsAndChildEntities")
    new_active_campaigns_and_child_entities: Optional[StrictBool] = Field(default=None, alias="NewActiveCampaignsAndChildEntities")
    new_ad_groups_and_child_entities_for_existing_campaigns: Optional[StrictBool] = Field(default=None, alias="NewAdGroupsAndChildEntitiesForExistingCampaigns")
    new_active_ads_for_existing_ad_groups: Optional[StrictBool] = Field(default=None, alias="NewActiveAdsForExistingAdGroups")
    new_keywords_for_existing_ad_groups: Optional[StrictBool] = Field(default=None, alias="NewKeywordsForExistingAdGroups")
    new_negative_keywords_for_existing_parents: Optional[StrictBool] = Field(default=None, alias="NewNegativeKeywordsForExistingParents")
    new_negative_keyword_lists: Optional[StrictBool] = Field(default=None, alias="NewNegativeKeywordLists")
    new_account_negative_keywords: Optional[StrictBool] = Field(default=None, alias="NewAccountNegativeKeywords")
    pause_new_campaigns: Optional[StrictBool] = Field(default=None, alias="PauseNewCampaigns")
    new_keyword_urls: Optional[StrictBool] = Field(default=None, alias="NewKeywordUrls")
    new_url_options: Optional[StrictBool] = Field(default=None, alias="NewUrlOptions")
    new_negative_sites: Optional[StrictBool] = Field(default=None, alias="NewNegativeSites")
    new_paused_ads_for_existing_ad_groups: Optional[StrictBool] = Field(default=None, alias="NewPausedAdsForExistingAdGroups")
    new_labels: Optional[StrictBool] = Field(default=None, alias="NewLabels")
    new_product_filters: Optional[StrictBool] = Field(default=None, alias="NewProductFilters")
    new_location_targets: Optional[StrictBool] = Field(default=None, alias="NewLocationTargets")
    new_ad_schedules: Optional[StrictBool] = Field(default=None, alias="NewAdSchedules")
    new_device_targets: Optional[StrictBool] = Field(default=None, alias="NewDeviceTargets")
    new_demographic_targets: Optional[StrictBool] = Field(default=None, alias="NewDemographicTargets")
    new_audience_targets: Optional[StrictBool] = Field(default=None, alias="NewAudienceTargets")
    new_app_ad_extensions: Optional[StrictBool] = Field(default=None, alias="NewAppAdExtensions")
    new_call_ad_extensions: Optional[StrictBool] = Field(default=None, alias="NewCallAdExtensions")
    new_callout_ad_extensions: Optional[StrictBool] = Field(default=None, alias="NewCalloutAdExtensions")
    new_price_ad_extensions: Optional[StrictBool] = Field(default=None, alias="NewPriceAdExtensions")
    new_location_ad_extensions: Optional[StrictBool] = Field(default=None, alias="NewLocationAdExtensions")
    new_review_ad_extensions: Optional[StrictBool] = Field(default=None, alias="NewReviewAdExtensions")
    new_sitelink_ad_extensions: Optional[StrictBool] = Field(default=None, alias="NewSitelinkAdExtensions")
    new_structured_snippet_ad_extensions: Optional[StrictBool] = Field(default=None, alias="NewStructuredSnippetAdExtensions")
    new_promotion_ad_extensions: Optional[StrictBool] = Field(default=None, alias="NewPromotionAdExtensions")
    new_image_ad_extensions: Optional[StrictBool] = Field(default=None, alias="NewImageAdExtensions")
    new_logo_ad_extensions: Optional[StrictBool] = Field(default=None, alias="NewLogoAdExtensions")
    new_lead_form_ad_extensions: Optional[StrictBool] = Field(default=None, alias="NewLeadFormAdExtensions")
    new_page_feeds: Optional[StrictBool] = Field(default=None, alias="NewPageFeeds")
    new_ad_customizer_feeds: Optional[StrictBool] = Field(default=None, alias="NewAdCustomizerFeeds")
    new_brand_suitability: Optional[StrictBool] = Field(default=None, alias="NewBrandSuitability")
    new_conversion_goals: Optional[StrictBool] = Field(default=None, alias="NewConversionGoals")
    new_carousel_ad: Optional[StrictBool] = Field(default=None, alias="NewCarouselAd")
    update_entities: Optional[StrictBool] = Field(default=None, alias="UpdateEntities")
    update_campaign_budgets: Optional[StrictBool] = Field(default=None, alias="UpdateCampaignBudgets")
    update_bids: Optional[StrictBool] = Field(default=None, alias="UpdateBids")
    update_negative_keyword_lists: Optional[StrictBool] = Field(default=None, alias="UpdateNegativeKeywordLists")
    update_account_negative_keywords: Optional[StrictBool] = Field(default=None, alias="UpdateAccountNegativeKeywords")
    update_negative_sites: Optional[StrictBool] = Field(default=None, alias="UpdateNegativeSites")
    update_keyword_urls: Optional[StrictBool] = Field(default=None, alias="UpdateKeywordUrls")
    update_url_options: Optional[StrictBool] = Field(default=None, alias="UpdateUrlOptions")
    update_campaign_names: Optional[StrictBool] = Field(default=None, alias="UpdateCampaignNames")
    update_labels: Optional[StrictBool] = Field(default=None, alias="UpdateLabels")
    update_product_filters: Optional[StrictBool] = Field(default=None, alias="UpdateProductFilters")
    update_conversion_goals: Optional[StrictBool] = Field(default=None, alias="UpdateConversionGoals")
    update_location_targets: Optional[StrictBool] = Field(default=None, alias="UpdateLocationTargets")
    update_ad_schedules: Optional[StrictBool] = Field(default=None, alias="UpdateAdSchedules")
    update_device_targets: Optional[StrictBool] = Field(default=None, alias="UpdateDeviceTargets")
    update_demographic_targets: Optional[StrictBool] = Field(default=None, alias="UpdateDemographicTargets")
    update_campaign_ad_group_languages: Optional[StrictBool] = Field(default=None, alias="UpdateCampaignAdGroupLanguages")
    update_audience_targets: Optional[StrictBool] = Field(default=None, alias="UpdateAudienceTargets")
    update_app_ad_extensions: Optional[StrictBool] = Field(default=None, alias="UpdateAppAdExtensions")
    update_call_ad_extensions: Optional[StrictBool] = Field(default=None, alias="UpdateCallAdExtensions")
    update_callout_ad_extensions: Optional[StrictBool] = Field(default=None, alias="UpdateCalloutAdExtensions")
    update_price_ad_extensions: Optional[StrictBool] = Field(default=None, alias="UpdatePriceAdExtensions")
    update_location_ad_extensions: Optional[StrictBool] = Field(default=None, alias="UpdateLocationAdExtensions")
    update_review_ad_extensions: Optional[StrictBool] = Field(default=None, alias="UpdateReviewAdExtensions")
    update_sitelink_ad_extensions: Optional[StrictBool] = Field(default=None, alias="UpdateSitelinkAdExtensions")
    update_structured_snippet_ad_extensions: Optional[StrictBool] = Field(default=None, alias="UpdateStructuredSnippetAdExtensions")
    update_promotion_ad_extensions: Optional[StrictBool] = Field(default=None, alias="UpdatePromotionAdExtensions")
    update_image_ad_extensions: Optional[StrictBool] = Field(default=None, alias="UpdateImageAdExtensions")
    update_logo_ad_extensions: Optional[StrictBool] = Field(default=None, alias="UpdateLogoAdExtensions")
    update_lead_form_ad_extensions: Optional[StrictBool] = Field(default=None, alias="UpdateLeadFormAdExtensions")
    update_status_for_campaigns: Optional[StrictBool] = Field(default=None, alias="UpdateStatusForCampaigns")
    update_status_for_ad_groups: Optional[StrictBool] = Field(default=None, alias="UpdateStatusForAdGroups")
    update_status_for_ads: Optional[StrictBool] = Field(default=None, alias="UpdateStatusForAds")
    update_status_for_keywords: Optional[StrictBool] = Field(default=None, alias="UpdateStatusForKeywords")
    update_ad_group_network: Optional[StrictBool] = Field(default=None, alias="UpdateAdGroupNetwork")
    update_page_feeds: Optional[StrictBool] = Field(default=None, alias="UpdatePageFeeds")
    update_ad_customizer_feeds: Optional[StrictBool] = Field(default=None, alias="UpdateAdCustomizerFeeds")
    update_brand_suitability: Optional[StrictBool] = Field(default=None, alias="UpdateBrandSuitability")
    update_ad_customizer_attributes: Optional[StrictBool] = Field(default=None, alias="UpdateAdCustomizerAttributes")
    delete_removed_entities: Optional[StrictBool] = Field(default=None, alias="DeleteRemovedEntities")
    raise_bids_to_minimum: Optional[StrictBool] = Field(default=None, alias="RaiseBidsToMinimum")
    raise_campaign_budgets_to_minimum: Optional[StrictBool] = Field(default=None, alias="RaiseCampaignBudgetsToMinimum")
    adjustment_for_campaign_budgets: Optional[StrictFloat] = Field(default=None, alias="AdjustmentForCampaignBudgets")
    adjustment_for_bids: Optional[StrictFloat] = Field(default=None, alias="AdjustmentForBids")
    suffix_for_urls: Optional[StrictStr] = Field(default=None, alias="SuffixForUrls")
    search_and_replace_for_urls: Optional[ImportSearchAndReplaceForStringProperty] = Field(default=None, alias="SearchAndReplaceForUrls")
    suffix_for_tracking_templates: Optional[StrictStr] = Field(default=None, alias="SuffixForTrackingTemplates")
    search_and_replace_for_tracking_templates: Optional[ImportSearchAndReplaceForStringProperty] = Field(default=None, alias="SearchAndReplaceForTrackingTemplates")
    suffix_for_campaign_names: Optional[StrictStr] = Field(default=None, alias="SuffixForCampaignNames")
    search_and_replace_for_campaign_names: Optional[ImportSearchAndReplaceForStringProperty] = Field(default=None, alias="SearchAndReplaceForCampaignNames")
    search_and_replace_for_custom_parameters: Optional[ImportSearchAndReplaceForStringProperty] = Field(default=None, alias="SearchAndReplaceForCustomParameters")
    search_and_replace_for_final_url_suffix: Optional[ImportSearchAndReplaceForStringProperty] = Field(default=None, alias="SearchAndReplaceForFinalURLSuffix")
    pause_campaigns_without_supported_locations: Optional[StrictBool] = Field(default=None, alias="PauseCampaignsWithoutSupportedLocations")
    enable_parent_location_mapping: Optional[StrictBool] = Field(default=None, alias="EnableParentLocationMapping")
    associated_store_id: Optional[StrictStr] = Field(default=None, alias="AssociatedStoreId")
    raise_product_group_bids_to_minimum: Optional[StrictBool] = Field(default=None, alias="RaiseProductGroupBidsToMinimum")
    search_and_dsa_mixed_campaign_as_search_campaign: Optional[StrictBool] = Field(default=None, alias="SearchAndDsaMixedCampaignAsSearchCampaign")
    account_url_options: Optional[StrictBool] = Field(default=None, alias="AccountUrlOptions")
    enable_auto_currency_conversion: Optional[StrictBool] = Field(default=None, alias="EnableAutoCurrencyConversion")
    update_bidding_strategies: Optional[StrictBool] = Field(default=None, alias="UpdateBiddingStrategies")
    associated_uet_tag_id: Optional[StrictStr] = Field(default=None, alias="AssociatedUetTagId")
    auto_device_bid_optimization: Optional[StrictBool] = Field(default=None, alias="AutoDeviceBidOptimization")
    ad_schedule_use_searcher_timezone: Optional[StrictBool] = Field(default=None, alias="AdScheduleUseSearcherTimezone")
    pause_aimad_group_if_all_audience_criterion_not_imported: Optional[StrictBool] = Field(default=None, alias="PauseAIMAdGroupIfAllAudienceCriterionNotImported")
    rename_campaign_name_with_suffix: Optional[StrictBool] = Field(default=None, alias="RenameCampaignNameWithSuffix")
    update_ad_urls: Optional[StrictBool] = Field(default=None, alias="UpdateAdUrls")
    update_sitelink_urls: Optional[StrictBool] = Field(default=None, alias="UpdateSitelinkUrls")
    forward_compatibility_map: Optional[List[Optional[KeyValuePairOfstringAndstring]]] = Field(default=None, alias="ForwardCompatibilityMap")
    type: Optional[StrictStr] = Field(default='GoogleImportOption', alias="Type")
    __properties: ClassVar[List[str]] = ["NewEntities", "NewPausedCampaignsAndChildEntities", "NewActiveCampaignsAndChildEntities", "NewAdGroupsAndChildEntitiesForExistingCampaigns", "NewActiveAdsForExistingAdGroups", "NewKeywordsForExistingAdGroups", "NewNegativeKeywordsForExistingParents", "NewNegativeKeywordLists", "NewAccountNegativeKeywords", "PauseNewCampaigns", "NewKeywordUrls", "NewUrlOptions", "NewNegativeSites", "NewPausedAdsForExistingAdGroups", "NewLabels", "NewProductFilters", "NewLocationTargets", "NewAdSchedules", "NewDeviceTargets", "NewDemographicTargets", "NewAudienceTargets", "NewAppAdExtensions", "NewCallAdExtensions", "NewCalloutAdExtensions", "NewPriceAdExtensions", "NewLocationAdExtensions", "NewReviewAdExtensions", "NewSitelinkAdExtensions", "NewStructuredSnippetAdExtensions", "NewPromotionAdExtensions", "NewImageAdExtensions", "NewLogoAdExtensions", "NewLeadFormAdExtensions", "NewPageFeeds", "NewAdCustomizerFeeds", "NewBrandSuitability", "NewConversionGoals", "NewCarouselAd", "UpdateEntities", "UpdateCampaignBudgets", "UpdateBids", "UpdateNegativeKeywordLists", "UpdateAccountNegativeKeywords", "UpdateNegativeSites", "UpdateKeywordUrls", "UpdateUrlOptions", "UpdateCampaignNames", "UpdateLabels", "UpdateProductFilters", "UpdateConversionGoals", "UpdateLocationTargets", "UpdateAdSchedules", "UpdateDeviceTargets", "UpdateDemographicTargets", "UpdateCampaignAdGroupLanguages", "UpdateAudienceTargets", "UpdateAppAdExtensions", "UpdateCallAdExtensions", "UpdateCalloutAdExtensions", "UpdatePriceAdExtensions", "UpdateLocationAdExtensions", "UpdateReviewAdExtensions", "UpdateSitelinkAdExtensions", "UpdateStructuredSnippetAdExtensions", "UpdatePromotionAdExtensions", "UpdateImageAdExtensions", "UpdateLogoAdExtensions", "UpdateLeadFormAdExtensions", "UpdateStatusForCampaigns", "UpdateStatusForAdGroups", "UpdateStatusForAds", "UpdateStatusForKeywords", "UpdateAdGroupNetwork", "UpdatePageFeeds", "UpdateAdCustomizerFeeds", "UpdateBrandSuitability", "UpdateAdCustomizerAttributes", "DeleteRemovedEntities", "RaiseBidsToMinimum", "RaiseCampaignBudgetsToMinimum", "AdjustmentForCampaignBudgets", "AdjustmentForBids", "SuffixForUrls", "SearchAndReplaceForUrls", "SuffixForTrackingTemplates", "SearchAndReplaceForTrackingTemplates", "SuffixForCampaignNames", "SearchAndReplaceForCampaignNames", "SearchAndReplaceForCustomParameters", "SearchAndReplaceForFinalURLSuffix", "PauseCampaignsWithoutSupportedLocations", "EnableParentLocationMapping", "AssociatedStoreId", "RaiseProductGroupBidsToMinimum", "SearchAndDsaMixedCampaignAsSearchCampaign", "AccountUrlOptions", "EnableAutoCurrencyConversion", "UpdateBiddingStrategies", "AssociatedUetTagId", "AutoDeviceBidOptimization", "AdScheduleUseSearcherTimezone", "PauseAIMAdGroupIfAllAudienceCriterionNotImported", "RenameCampaignNameWithSuffix", "UpdateAdUrls", "UpdateSitelinkUrls", "ForwardCompatibilityMap", "Type"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	
    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())
	
    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GoogleImportOption from a JSON string"""
        return cls.from_dict(json.loads(json_str))
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of search_and_replace_for_urls
        if self.search_and_replace_for_urls:
            _dict['SearchAndReplaceForUrls'] = self.search_and_replace_for_urls.to_dict()
        # override the default output from pydantic by calling `to_dict()` of search_and_replace_for_tracking_templates
        if self.search_and_replace_for_tracking_templates:
            _dict['SearchAndReplaceForTrackingTemplates'] = self.search_and_replace_for_tracking_templates.to_dict()
        # override the default output from pydantic by calling `to_dict()` of search_and_replace_for_campaign_names
        if self.search_and_replace_for_campaign_names:
            _dict['SearchAndReplaceForCampaignNames'] = self.search_and_replace_for_campaign_names.to_dict()
        # override the default output from pydantic by calling `to_dict()` of search_and_replace_for_custom_parameters
        if self.search_and_replace_for_custom_parameters:
            _dict['SearchAndReplaceForCustomParameters'] = self.search_and_replace_for_custom_parameters.to_dict()
        # override the default output from pydantic by calling `to_dict()` of search_and_replace_for_final_url_suffix
        if self.search_and_replace_for_final_url_suffix:
            _dict['SearchAndReplaceForFinalURLSuffix'] = self.search_and_replace_for_final_url_suffix.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in forward_compatibility_map (list)
        _items = []
        if self.forward_compatibility_map:
            for _item_forward_compatibility_map in self.forward_compatibility_map:
                if _item_forward_compatibility_map:
                    _items.append(_item_forward_compatibility_map.to_dict())
            _dict['ForwardCompatibilityMap'] = _items
        # set to None if new_entities (nullable) is None
        # and model_fields_set contains the field
        if self.new_entities is None and "new_entities" in self.model_fields_set:
            _dict['NewEntities'] = None

        # set to None if new_paused_campaigns_and_child_entities (nullable) is None
        # and model_fields_set contains the field
        if self.new_paused_campaigns_and_child_entities is None and "new_paused_campaigns_and_child_entities" in self.model_fields_set:
            _dict['NewPausedCampaignsAndChildEntities'] = None

        # set to None if new_active_campaigns_and_child_entities (nullable) is None
        # and model_fields_set contains the field
        if self.new_active_campaigns_and_child_entities is None and "new_active_campaigns_and_child_entities" in self.model_fields_set:
            _dict['NewActiveCampaignsAndChildEntities'] = None

        # set to None if new_ad_groups_and_child_entities_for_existing_campaigns (nullable) is None
        # and model_fields_set contains the field
        if self.new_ad_groups_and_child_entities_for_existing_campaigns is None and "new_ad_groups_and_child_entities_for_existing_campaigns" in self.model_fields_set:
            _dict['NewAdGroupsAndChildEntitiesForExistingCampaigns'] = None

        # set to None if new_active_ads_for_existing_ad_groups (nullable) is None
        # and model_fields_set contains the field
        if self.new_active_ads_for_existing_ad_groups is None and "new_active_ads_for_existing_ad_groups" in self.model_fields_set:
            _dict['NewActiveAdsForExistingAdGroups'] = None

        # set to None if new_keywords_for_existing_ad_groups (nullable) is None
        # and model_fields_set contains the field
        if self.new_keywords_for_existing_ad_groups is None and "new_keywords_for_existing_ad_groups" in self.model_fields_set:
            _dict['NewKeywordsForExistingAdGroups'] = None

        # set to None if new_negative_keywords_for_existing_parents (nullable) is None
        # and model_fields_set contains the field
        if self.new_negative_keywords_for_existing_parents is None and "new_negative_keywords_for_existing_parents" in self.model_fields_set:
            _dict['NewNegativeKeywordsForExistingParents'] = None

        # set to None if new_negative_keyword_lists (nullable) is None
        # and model_fields_set contains the field
        if self.new_negative_keyword_lists is None and "new_negative_keyword_lists" in self.model_fields_set:
            _dict['NewNegativeKeywordLists'] = None

        # set to None if new_account_negative_keywords (nullable) is None
        # and model_fields_set contains the field
        if self.new_account_negative_keywords is None and "new_account_negative_keywords" in self.model_fields_set:
            _dict['NewAccountNegativeKeywords'] = None

        # set to None if pause_new_campaigns (nullable) is None
        # and model_fields_set contains the field
        if self.pause_new_campaigns is None and "pause_new_campaigns" in self.model_fields_set:
            _dict['PauseNewCampaigns'] = None

        # set to None if new_keyword_urls (nullable) is None
        # and model_fields_set contains the field
        if self.new_keyword_urls is None and "new_keyword_urls" in self.model_fields_set:
            _dict['NewKeywordUrls'] = None

        # set to None if new_url_options (nullable) is None
        # and model_fields_set contains the field
        if self.new_url_options is None and "new_url_options" in self.model_fields_set:
            _dict['NewUrlOptions'] = None

        # set to None if new_negative_sites (nullable) is None
        # and model_fields_set contains the field
        if self.new_negative_sites is None and "new_negative_sites" in self.model_fields_set:
            _dict['NewNegativeSites'] = None

        # set to None if new_paused_ads_for_existing_ad_groups (nullable) is None
        # and model_fields_set contains the field
        if self.new_paused_ads_for_existing_ad_groups is None and "new_paused_ads_for_existing_ad_groups" in self.model_fields_set:
            _dict['NewPausedAdsForExistingAdGroups'] = None

        # set to None if new_labels (nullable) is None
        # and model_fields_set contains the field
        if self.new_labels is None and "new_labels" in self.model_fields_set:
            _dict['NewLabels'] = None

        # set to None if new_product_filters (nullable) is None
        # and model_fields_set contains the field
        if self.new_product_filters is None and "new_product_filters" in self.model_fields_set:
            _dict['NewProductFilters'] = None

        # set to None if new_location_targets (nullable) is None
        # and model_fields_set contains the field
        if self.new_location_targets is None and "new_location_targets" in self.model_fields_set:
            _dict['NewLocationTargets'] = None

        # set to None if new_ad_schedules (nullable) is None
        # and model_fields_set contains the field
        if self.new_ad_schedules is None and "new_ad_schedules" in self.model_fields_set:
            _dict['NewAdSchedules'] = None

        # set to None if new_device_targets (nullable) is None
        # and model_fields_set contains the field
        if self.new_device_targets is None and "new_device_targets" in self.model_fields_set:
            _dict['NewDeviceTargets'] = None

        # set to None if new_demographic_targets (nullable) is None
        # and model_fields_set contains the field
        if self.new_demographic_targets is None and "new_demographic_targets" in self.model_fields_set:
            _dict['NewDemographicTargets'] = None

        # set to None if new_audience_targets (nullable) is None
        # and model_fields_set contains the field
        if self.new_audience_targets is None and "new_audience_targets" in self.model_fields_set:
            _dict['NewAudienceTargets'] = None

        # set to None if new_app_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.new_app_ad_extensions is None and "new_app_ad_extensions" in self.model_fields_set:
            _dict['NewAppAdExtensions'] = None

        # set to None if new_call_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.new_call_ad_extensions is None and "new_call_ad_extensions" in self.model_fields_set:
            _dict['NewCallAdExtensions'] = None

        # set to None if new_callout_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.new_callout_ad_extensions is None and "new_callout_ad_extensions" in self.model_fields_set:
            _dict['NewCalloutAdExtensions'] = None

        # set to None if new_price_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.new_price_ad_extensions is None and "new_price_ad_extensions" in self.model_fields_set:
            _dict['NewPriceAdExtensions'] = None

        # set to None if new_location_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.new_location_ad_extensions is None and "new_location_ad_extensions" in self.model_fields_set:
            _dict['NewLocationAdExtensions'] = None

        # set to None if new_review_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.new_review_ad_extensions is None and "new_review_ad_extensions" in self.model_fields_set:
            _dict['NewReviewAdExtensions'] = None

        # set to None if new_sitelink_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.new_sitelink_ad_extensions is None and "new_sitelink_ad_extensions" in self.model_fields_set:
            _dict['NewSitelinkAdExtensions'] = None

        # set to None if new_structured_snippet_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.new_structured_snippet_ad_extensions is None and "new_structured_snippet_ad_extensions" in self.model_fields_set:
            _dict['NewStructuredSnippetAdExtensions'] = None

        # set to None if new_promotion_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.new_promotion_ad_extensions is None and "new_promotion_ad_extensions" in self.model_fields_set:
            _dict['NewPromotionAdExtensions'] = None

        # set to None if new_image_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.new_image_ad_extensions is None and "new_image_ad_extensions" in self.model_fields_set:
            _dict['NewImageAdExtensions'] = None

        # set to None if new_logo_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.new_logo_ad_extensions is None and "new_logo_ad_extensions" in self.model_fields_set:
            _dict['NewLogoAdExtensions'] = None

        # set to None if new_lead_form_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.new_lead_form_ad_extensions is None and "new_lead_form_ad_extensions" in self.model_fields_set:
            _dict['NewLeadFormAdExtensions'] = None

        # set to None if new_page_feeds (nullable) is None
        # and model_fields_set contains the field
        if self.new_page_feeds is None and "new_page_feeds" in self.model_fields_set:
            _dict['NewPageFeeds'] = None

        # set to None if new_ad_customizer_feeds (nullable) is None
        # and model_fields_set contains the field
        if self.new_ad_customizer_feeds is None and "new_ad_customizer_feeds" in self.model_fields_set:
            _dict['NewAdCustomizerFeeds'] = None

        # set to None if new_brand_suitability (nullable) is None
        # and model_fields_set contains the field
        if self.new_brand_suitability is None and "new_brand_suitability" in self.model_fields_set:
            _dict['NewBrandSuitability'] = None

        # set to None if new_conversion_goals (nullable) is None
        # and model_fields_set contains the field
        if self.new_conversion_goals is None and "new_conversion_goals" in self.model_fields_set:
            _dict['NewConversionGoals'] = None

        # set to None if new_carousel_ad (nullable) is None
        # and model_fields_set contains the field
        if self.new_carousel_ad is None and "new_carousel_ad" in self.model_fields_set:
            _dict['NewCarouselAd'] = None

        # set to None if update_entities (nullable) is None
        # and model_fields_set contains the field
        if self.update_entities is None and "update_entities" in self.model_fields_set:
            _dict['UpdateEntities'] = None

        # set to None if update_campaign_budgets (nullable) is None
        # and model_fields_set contains the field
        if self.update_campaign_budgets is None and "update_campaign_budgets" in self.model_fields_set:
            _dict['UpdateCampaignBudgets'] = None

        # set to None if update_bids (nullable) is None
        # and model_fields_set contains the field
        if self.update_bids is None and "update_bids" in self.model_fields_set:
            _dict['UpdateBids'] = None

        # set to None if update_negative_keyword_lists (nullable) is None
        # and model_fields_set contains the field
        if self.update_negative_keyword_lists is None and "update_negative_keyword_lists" in self.model_fields_set:
            _dict['UpdateNegativeKeywordLists'] = None

        # set to None if update_account_negative_keywords (nullable) is None
        # and model_fields_set contains the field
        if self.update_account_negative_keywords is None and "update_account_negative_keywords" in self.model_fields_set:
            _dict['UpdateAccountNegativeKeywords'] = None

        # set to None if update_negative_sites (nullable) is None
        # and model_fields_set contains the field
        if self.update_negative_sites is None and "update_negative_sites" in self.model_fields_set:
            _dict['UpdateNegativeSites'] = None

        # set to None if update_keyword_urls (nullable) is None
        # and model_fields_set contains the field
        if self.update_keyword_urls is None and "update_keyword_urls" in self.model_fields_set:
            _dict['UpdateKeywordUrls'] = None

        # set to None if update_url_options (nullable) is None
        # and model_fields_set contains the field
        if self.update_url_options is None and "update_url_options" in self.model_fields_set:
            _dict['UpdateUrlOptions'] = None

        # set to None if update_campaign_names (nullable) is None
        # and model_fields_set contains the field
        if self.update_campaign_names is None and "update_campaign_names" in self.model_fields_set:
            _dict['UpdateCampaignNames'] = None

        # set to None if update_labels (nullable) is None
        # and model_fields_set contains the field
        if self.update_labels is None and "update_labels" in self.model_fields_set:
            _dict['UpdateLabels'] = None

        # set to None if update_product_filters (nullable) is None
        # and model_fields_set contains the field
        if self.update_product_filters is None and "update_product_filters" in self.model_fields_set:
            _dict['UpdateProductFilters'] = None

        # set to None if update_conversion_goals (nullable) is None
        # and model_fields_set contains the field
        if self.update_conversion_goals is None and "update_conversion_goals" in self.model_fields_set:
            _dict['UpdateConversionGoals'] = None

        # set to None if update_location_targets (nullable) is None
        # and model_fields_set contains the field
        if self.update_location_targets is None and "update_location_targets" in self.model_fields_set:
            _dict['UpdateLocationTargets'] = None

        # set to None if update_ad_schedules (nullable) is None
        # and model_fields_set contains the field
        if self.update_ad_schedules is None and "update_ad_schedules" in self.model_fields_set:
            _dict['UpdateAdSchedules'] = None

        # set to None if update_device_targets (nullable) is None
        # and model_fields_set contains the field
        if self.update_device_targets is None and "update_device_targets" in self.model_fields_set:
            _dict['UpdateDeviceTargets'] = None

        # set to None if update_demographic_targets (nullable) is None
        # and model_fields_set contains the field
        if self.update_demographic_targets is None and "update_demographic_targets" in self.model_fields_set:
            _dict['UpdateDemographicTargets'] = None

        # set to None if update_campaign_ad_group_languages (nullable) is None
        # and model_fields_set contains the field
        if self.update_campaign_ad_group_languages is None and "update_campaign_ad_group_languages" in self.model_fields_set:
            _dict['UpdateCampaignAdGroupLanguages'] = None

        # set to None if update_audience_targets (nullable) is None
        # and model_fields_set contains the field
        if self.update_audience_targets is None and "update_audience_targets" in self.model_fields_set:
            _dict['UpdateAudienceTargets'] = None

        # set to None if update_app_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.update_app_ad_extensions is None and "update_app_ad_extensions" in self.model_fields_set:
            _dict['UpdateAppAdExtensions'] = None

        # set to None if update_call_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.update_call_ad_extensions is None and "update_call_ad_extensions" in self.model_fields_set:
            _dict['UpdateCallAdExtensions'] = None

        # set to None if update_callout_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.update_callout_ad_extensions is None and "update_callout_ad_extensions" in self.model_fields_set:
            _dict['UpdateCalloutAdExtensions'] = None

        # set to None if update_price_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.update_price_ad_extensions is None and "update_price_ad_extensions" in self.model_fields_set:
            _dict['UpdatePriceAdExtensions'] = None

        # set to None if update_location_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.update_location_ad_extensions is None and "update_location_ad_extensions" in self.model_fields_set:
            _dict['UpdateLocationAdExtensions'] = None

        # set to None if update_review_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.update_review_ad_extensions is None and "update_review_ad_extensions" in self.model_fields_set:
            _dict['UpdateReviewAdExtensions'] = None

        # set to None if update_sitelink_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.update_sitelink_ad_extensions is None and "update_sitelink_ad_extensions" in self.model_fields_set:
            _dict['UpdateSitelinkAdExtensions'] = None

        # set to None if update_structured_snippet_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.update_structured_snippet_ad_extensions is None and "update_structured_snippet_ad_extensions" in self.model_fields_set:
            _dict['UpdateStructuredSnippetAdExtensions'] = None

        # set to None if update_promotion_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.update_promotion_ad_extensions is None and "update_promotion_ad_extensions" in self.model_fields_set:
            _dict['UpdatePromotionAdExtensions'] = None

        # set to None if update_image_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.update_image_ad_extensions is None and "update_image_ad_extensions" in self.model_fields_set:
            _dict['UpdateImageAdExtensions'] = None

        # set to None if update_logo_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.update_logo_ad_extensions is None and "update_logo_ad_extensions" in self.model_fields_set:
            _dict['UpdateLogoAdExtensions'] = None

        # set to None if update_lead_form_ad_extensions (nullable) is None
        # and model_fields_set contains the field
        if self.update_lead_form_ad_extensions is None and "update_lead_form_ad_extensions" in self.model_fields_set:
            _dict['UpdateLeadFormAdExtensions'] = None

        # set to None if update_status_for_campaigns (nullable) is None
        # and model_fields_set contains the field
        if self.update_status_for_campaigns is None and "update_status_for_campaigns" in self.model_fields_set:
            _dict['UpdateStatusForCampaigns'] = None

        # set to None if update_status_for_ad_groups (nullable) is None
        # and model_fields_set contains the field
        if self.update_status_for_ad_groups is None and "update_status_for_ad_groups" in self.model_fields_set:
            _dict['UpdateStatusForAdGroups'] = None

        # set to None if update_status_for_ads (nullable) is None
        # and model_fields_set contains the field
        if self.update_status_for_ads is None and "update_status_for_ads" in self.model_fields_set:
            _dict['UpdateStatusForAds'] = None

        # set to None if update_status_for_keywords (nullable) is None
        # and model_fields_set contains the field
        if self.update_status_for_keywords is None and "update_status_for_keywords" in self.model_fields_set:
            _dict['UpdateStatusForKeywords'] = None

        # set to None if update_ad_group_network (nullable) is None
        # and model_fields_set contains the field
        if self.update_ad_group_network is None and "update_ad_group_network" in self.model_fields_set:
            _dict['UpdateAdGroupNetwork'] = None

        # set to None if update_page_feeds (nullable) is None
        # and model_fields_set contains the field
        if self.update_page_feeds is None and "update_page_feeds" in self.model_fields_set:
            _dict['UpdatePageFeeds'] = None

        # set to None if update_ad_customizer_feeds (nullable) is None
        # and model_fields_set contains the field
        if self.update_ad_customizer_feeds is None and "update_ad_customizer_feeds" in self.model_fields_set:
            _dict['UpdateAdCustomizerFeeds'] = None

        # set to None if update_brand_suitability (nullable) is None
        # and model_fields_set contains the field
        if self.update_brand_suitability is None and "update_brand_suitability" in self.model_fields_set:
            _dict['UpdateBrandSuitability'] = None

        # set to None if update_ad_customizer_attributes (nullable) is None
        # and model_fields_set contains the field
        if self.update_ad_customizer_attributes is None and "update_ad_customizer_attributes" in self.model_fields_set:
            _dict['UpdateAdCustomizerAttributes'] = None

        # set to None if delete_removed_entities (nullable) is None
        # and model_fields_set contains the field
        if self.delete_removed_entities is None and "delete_removed_entities" in self.model_fields_set:
            _dict['DeleteRemovedEntities'] = None

        # set to None if raise_bids_to_minimum (nullable) is None
        # and model_fields_set contains the field
        if self.raise_bids_to_minimum is None and "raise_bids_to_minimum" in self.model_fields_set:
            _dict['RaiseBidsToMinimum'] = None

        # set to None if raise_campaign_budgets_to_minimum (nullable) is None
        # and model_fields_set contains the field
        if self.raise_campaign_budgets_to_minimum is None and "raise_campaign_budgets_to_minimum" in self.model_fields_set:
            _dict['RaiseCampaignBudgetsToMinimum'] = None

        # set to None if adjustment_for_campaign_budgets (nullable) is None
        # and model_fields_set contains the field
        if self.adjustment_for_campaign_budgets is None and "adjustment_for_campaign_budgets" in self.model_fields_set:
            _dict['AdjustmentForCampaignBudgets'] = None

        # set to None if adjustment_for_bids (nullable) is None
        # and model_fields_set contains the field
        if self.adjustment_for_bids is None and "adjustment_for_bids" in self.model_fields_set:
            _dict['AdjustmentForBids'] = None

        # set to None if suffix_for_urls (nullable) is None
        # and model_fields_set contains the field
        if self.suffix_for_urls is None and "suffix_for_urls" in self.model_fields_set:
            _dict['SuffixForUrls'] = None

        # set to None if search_and_replace_for_urls (nullable) is None
        # and model_fields_set contains the field
        if self.search_and_replace_for_urls is None and "search_and_replace_for_urls" in self.model_fields_set:
            _dict['SearchAndReplaceForUrls'] = None

        # set to None if suffix_for_tracking_templates (nullable) is None
        # and model_fields_set contains the field
        if self.suffix_for_tracking_templates is None and "suffix_for_tracking_templates" in self.model_fields_set:
            _dict['SuffixForTrackingTemplates'] = None

        # set to None if search_and_replace_for_tracking_templates (nullable) is None
        # and model_fields_set contains the field
        if self.search_and_replace_for_tracking_templates is None and "search_and_replace_for_tracking_templates" in self.model_fields_set:
            _dict['SearchAndReplaceForTrackingTemplates'] = None

        # set to None if suffix_for_campaign_names (nullable) is None
        # and model_fields_set contains the field
        if self.suffix_for_campaign_names is None and "suffix_for_campaign_names" in self.model_fields_set:
            _dict['SuffixForCampaignNames'] = None

        # set to None if search_and_replace_for_campaign_names (nullable) is None
        # and model_fields_set contains the field
        if self.search_and_replace_for_campaign_names is None and "search_and_replace_for_campaign_names" in self.model_fields_set:
            _dict['SearchAndReplaceForCampaignNames'] = None

        # set to None if search_and_replace_for_custom_parameters (nullable) is None
        # and model_fields_set contains the field
        if self.search_and_replace_for_custom_parameters is None and "search_and_replace_for_custom_parameters" in self.model_fields_set:
            _dict['SearchAndReplaceForCustomParameters'] = None

        # set to None if search_and_replace_for_final_url_suffix (nullable) is None
        # and model_fields_set contains the field
        if self.search_and_replace_for_final_url_suffix is None and "search_and_replace_for_final_url_suffix" in self.model_fields_set:
            _dict['SearchAndReplaceForFinalURLSuffix'] = None

        # set to None if pause_campaigns_without_supported_locations (nullable) is None
        # and model_fields_set contains the field
        if self.pause_campaigns_without_supported_locations is None and "pause_campaigns_without_supported_locations" in self.model_fields_set:
            _dict['PauseCampaignsWithoutSupportedLocations'] = None

        # set to None if enable_parent_location_mapping (nullable) is None
        # and model_fields_set contains the field
        if self.enable_parent_location_mapping is None and "enable_parent_location_mapping" in self.model_fields_set:
            _dict['EnableParentLocationMapping'] = None

        # set to None if associated_store_id (nullable) is None
        # and model_fields_set contains the field
        if self.associated_store_id is None and "associated_store_id" in self.model_fields_set:
            _dict['AssociatedStoreId'] = None

        # set to None if raise_product_group_bids_to_minimum (nullable) is None
        # and model_fields_set contains the field
        if self.raise_product_group_bids_to_minimum is None and "raise_product_group_bids_to_minimum" in self.model_fields_set:
            _dict['RaiseProductGroupBidsToMinimum'] = None

        # set to None if search_and_dsa_mixed_campaign_as_search_campaign (nullable) is None
        # and model_fields_set contains the field
        if self.search_and_dsa_mixed_campaign_as_search_campaign is None and "search_and_dsa_mixed_campaign_as_search_campaign" in self.model_fields_set:
            _dict['SearchAndDsaMixedCampaignAsSearchCampaign'] = None

        # set to None if account_url_options (nullable) is None
        # and model_fields_set contains the field
        if self.account_url_options is None and "account_url_options" in self.model_fields_set:
            _dict['AccountUrlOptions'] = None

        # set to None if enable_auto_currency_conversion (nullable) is None
        # and model_fields_set contains the field
        if self.enable_auto_currency_conversion is None and "enable_auto_currency_conversion" in self.model_fields_set:
            _dict['EnableAutoCurrencyConversion'] = None

        # set to None if update_bidding_strategies (nullable) is None
        # and model_fields_set contains the field
        if self.update_bidding_strategies is None and "update_bidding_strategies" in self.model_fields_set:
            _dict['UpdateBiddingStrategies'] = None

        # set to None if associated_uet_tag_id (nullable) is None
        # and model_fields_set contains the field
        if self.associated_uet_tag_id is None and "associated_uet_tag_id" in self.model_fields_set:
            _dict['AssociatedUetTagId'] = None

        # set to None if auto_device_bid_optimization (nullable) is None
        # and model_fields_set contains the field
        if self.auto_device_bid_optimization is None and "auto_device_bid_optimization" in self.model_fields_set:
            _dict['AutoDeviceBidOptimization'] = None

        # set to None if ad_schedule_use_searcher_timezone (nullable) is None
        # and model_fields_set contains the field
        if self.ad_schedule_use_searcher_timezone is None and "ad_schedule_use_searcher_timezone" in self.model_fields_set:
            _dict['AdScheduleUseSearcherTimezone'] = None

        # set to None if pause_aimad_group_if_all_audience_criterion_not_imported (nullable) is None
        # and model_fields_set contains the field
        if self.pause_aimad_group_if_all_audience_criterion_not_imported is None and "pause_aimad_group_if_all_audience_criterion_not_imported" in self.model_fields_set:
            _dict['PauseAIMAdGroupIfAllAudienceCriterionNotImported'] = None

        # set to None if rename_campaign_name_with_suffix (nullable) is None
        # and model_fields_set contains the field
        if self.rename_campaign_name_with_suffix is None and "rename_campaign_name_with_suffix" in self.model_fields_set:
            _dict['RenameCampaignNameWithSuffix'] = None

        # set to None if update_ad_urls (nullable) is None
        # and model_fields_set contains the field
        if self.update_ad_urls is None and "update_ad_urls" in self.model_fields_set:
            _dict['UpdateAdUrls'] = None

        # set to None if update_sitelink_urls (nullable) is None
        # and model_fields_set contains the field
        if self.update_sitelink_urls is None and "update_sitelink_urls" in self.model_fields_set:
            _dict['UpdateSitelinkUrls'] = None

        # set to None if forward_compatibility_map (nullable) is None
        # and model_fields_set contains the field
        if self.forward_compatibility_map is None and "forward_compatibility_map" in self.model_fields_set:
            _dict['ForwardCompatibilityMap'] = None

        # set to None if type (nullable) is None
        # and model_fields_set contains the field
        if self.type is None and "type" in self.model_fields_set:
            _dict['Type'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GoogleImportOption from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "NewEntities": obj.get("NewEntities") if obj.get("NewEntities") is not None else None,
                        "NewPausedCampaignsAndChildEntities": obj.get("NewPausedCampaignsAndChildEntities") if obj.get("NewPausedCampaignsAndChildEntities") is not None else None,
                        "NewActiveCampaignsAndChildEntities": obj.get("NewActiveCampaignsAndChildEntities") if obj.get("NewActiveCampaignsAndChildEntities") is not None else None,
                        "NewAdGroupsAndChildEntitiesForExistingCampaigns": obj.get("NewAdGroupsAndChildEntitiesForExistingCampaigns") if obj.get("NewAdGroupsAndChildEntitiesForExistingCampaigns") is not None else None,
                        "NewActiveAdsForExistingAdGroups": obj.get("NewActiveAdsForExistingAdGroups") if obj.get("NewActiveAdsForExistingAdGroups") is not None else None,
                        "NewKeywordsForExistingAdGroups": obj.get("NewKeywordsForExistingAdGroups") if obj.get("NewKeywordsForExistingAdGroups") is not None else None,
                        "NewNegativeKeywordsForExistingParents": obj.get("NewNegativeKeywordsForExistingParents") if obj.get("NewNegativeKeywordsForExistingParents") is not None else None,
                        "NewNegativeKeywordLists": obj.get("NewNegativeKeywordLists") if obj.get("NewNegativeKeywordLists") is not None else None,
                        "NewAccountNegativeKeywords": obj.get("NewAccountNegativeKeywords") if obj.get("NewAccountNegativeKeywords") is not None else None,
                        "PauseNewCampaigns": obj.get("PauseNewCampaigns") if obj.get("PauseNewCampaigns") is not None else None,
                        "NewKeywordUrls": obj.get("NewKeywordUrls") if obj.get("NewKeywordUrls") is not None else None,
                        "NewUrlOptions": obj.get("NewUrlOptions") if obj.get("NewUrlOptions") is not None else None,
                        "NewNegativeSites": obj.get("NewNegativeSites") if obj.get("NewNegativeSites") is not None else None,
                        "NewPausedAdsForExistingAdGroups": obj.get("NewPausedAdsForExistingAdGroups") if obj.get("NewPausedAdsForExistingAdGroups") is not None else None,
                        "NewLabels": obj.get("NewLabels") if obj.get("NewLabels") is not None else None,
                        "NewProductFilters": obj.get("NewProductFilters") if obj.get("NewProductFilters") is not None else None,
                        "NewLocationTargets": obj.get("NewLocationTargets") if obj.get("NewLocationTargets") is not None else None,
                        "NewAdSchedules": obj.get("NewAdSchedules") if obj.get("NewAdSchedules") is not None else None,
                        "NewDeviceTargets": obj.get("NewDeviceTargets") if obj.get("NewDeviceTargets") is not None else None,
                        "NewDemographicTargets": obj.get("NewDemographicTargets") if obj.get("NewDemographicTargets") is not None else None,
                        "NewAudienceTargets": obj.get("NewAudienceTargets") if obj.get("NewAudienceTargets") is not None else None,
                        "NewAppAdExtensions": obj.get("NewAppAdExtensions") if obj.get("NewAppAdExtensions") is not None else None,
                        "NewCallAdExtensions": obj.get("NewCallAdExtensions") if obj.get("NewCallAdExtensions") is not None else None,
                        "NewCalloutAdExtensions": obj.get("NewCalloutAdExtensions") if obj.get("NewCalloutAdExtensions") is not None else None,
                        "NewPriceAdExtensions": obj.get("NewPriceAdExtensions") if obj.get("NewPriceAdExtensions") is not None else None,
                        "NewLocationAdExtensions": obj.get("NewLocationAdExtensions") if obj.get("NewLocationAdExtensions") is not None else None,
                        "NewReviewAdExtensions": obj.get("NewReviewAdExtensions") if obj.get("NewReviewAdExtensions") is not None else None,
                        "NewSitelinkAdExtensions": obj.get("NewSitelinkAdExtensions") if obj.get("NewSitelinkAdExtensions") is not None else None,
                        "NewStructuredSnippetAdExtensions": obj.get("NewStructuredSnippetAdExtensions") if obj.get("NewStructuredSnippetAdExtensions") is not None else None,
                        "NewPromotionAdExtensions": obj.get("NewPromotionAdExtensions") if obj.get("NewPromotionAdExtensions") is not None else None,
                        "NewImageAdExtensions": obj.get("NewImageAdExtensions") if obj.get("NewImageAdExtensions") is not None else None,
                        "NewLogoAdExtensions": obj.get("NewLogoAdExtensions") if obj.get("NewLogoAdExtensions") is not None else None,
                        "NewLeadFormAdExtensions": obj.get("NewLeadFormAdExtensions") if obj.get("NewLeadFormAdExtensions") is not None else None,
                        "NewPageFeeds": obj.get("NewPageFeeds") if obj.get("NewPageFeeds") is not None else None,
                        "NewAdCustomizerFeeds": obj.get("NewAdCustomizerFeeds") if obj.get("NewAdCustomizerFeeds") is not None else None,
                        "NewBrandSuitability": obj.get("NewBrandSuitability") if obj.get("NewBrandSuitability") is not None else None,
                        "NewConversionGoals": obj.get("NewConversionGoals") if obj.get("NewConversionGoals") is not None else None,
                        "NewCarouselAd": obj.get("NewCarouselAd") if obj.get("NewCarouselAd") is not None else None,
                        "UpdateEntities": obj.get("UpdateEntities") if obj.get("UpdateEntities") is not None else None,
                        "UpdateCampaignBudgets": obj.get("UpdateCampaignBudgets") if obj.get("UpdateCampaignBudgets") is not None else None,
                        "UpdateBids": obj.get("UpdateBids") if obj.get("UpdateBids") is not None else None,
                        "UpdateNegativeKeywordLists": obj.get("UpdateNegativeKeywordLists") if obj.get("UpdateNegativeKeywordLists") is not None else None,
                        "UpdateAccountNegativeKeywords": obj.get("UpdateAccountNegativeKeywords") if obj.get("UpdateAccountNegativeKeywords") is not None else None,
                        "UpdateNegativeSites": obj.get("UpdateNegativeSites") if obj.get("UpdateNegativeSites") is not None else None,
                        "UpdateKeywordUrls": obj.get("UpdateKeywordUrls") if obj.get("UpdateKeywordUrls") is not None else None,
                        "UpdateUrlOptions": obj.get("UpdateUrlOptions") if obj.get("UpdateUrlOptions") is not None else None,
                        "UpdateCampaignNames": obj.get("UpdateCampaignNames") if obj.get("UpdateCampaignNames") is not None else None,
                        "UpdateLabels": obj.get("UpdateLabels") if obj.get("UpdateLabels") is not None else None,
                        "UpdateProductFilters": obj.get("UpdateProductFilters") if obj.get("UpdateProductFilters") is not None else None,
                        "UpdateConversionGoals": obj.get("UpdateConversionGoals") if obj.get("UpdateConversionGoals") is not None else None,
                        "UpdateLocationTargets": obj.get("UpdateLocationTargets") if obj.get("UpdateLocationTargets") is not None else None,
                        "UpdateAdSchedules": obj.get("UpdateAdSchedules") if obj.get("UpdateAdSchedules") is not None else None,
                        "UpdateDeviceTargets": obj.get("UpdateDeviceTargets") if obj.get("UpdateDeviceTargets") is not None else None,
                        "UpdateDemographicTargets": obj.get("UpdateDemographicTargets") if obj.get("UpdateDemographicTargets") is not None else None,
                        "UpdateCampaignAdGroupLanguages": obj.get("UpdateCampaignAdGroupLanguages") if obj.get("UpdateCampaignAdGroupLanguages") is not None else None,
                        "UpdateAudienceTargets": obj.get("UpdateAudienceTargets") if obj.get("UpdateAudienceTargets") is not None else None,
                        "UpdateAppAdExtensions": obj.get("UpdateAppAdExtensions") if obj.get("UpdateAppAdExtensions") is not None else None,
                        "UpdateCallAdExtensions": obj.get("UpdateCallAdExtensions") if obj.get("UpdateCallAdExtensions") is not None else None,
                        "UpdateCalloutAdExtensions": obj.get("UpdateCalloutAdExtensions") if obj.get("UpdateCalloutAdExtensions") is not None else None,
                        "UpdatePriceAdExtensions": obj.get("UpdatePriceAdExtensions") if obj.get("UpdatePriceAdExtensions") is not None else None,
                        "UpdateLocationAdExtensions": obj.get("UpdateLocationAdExtensions") if obj.get("UpdateLocationAdExtensions") is not None else None,
                        "UpdateReviewAdExtensions": obj.get("UpdateReviewAdExtensions") if obj.get("UpdateReviewAdExtensions") is not None else None,
                        "UpdateSitelinkAdExtensions": obj.get("UpdateSitelinkAdExtensions") if obj.get("UpdateSitelinkAdExtensions") is not None else None,
                        "UpdateStructuredSnippetAdExtensions": obj.get("UpdateStructuredSnippetAdExtensions") if obj.get("UpdateStructuredSnippetAdExtensions") is not None else None,
                        "UpdatePromotionAdExtensions": obj.get("UpdatePromotionAdExtensions") if obj.get("UpdatePromotionAdExtensions") is not None else None,
                        "UpdateImageAdExtensions": obj.get("UpdateImageAdExtensions") if obj.get("UpdateImageAdExtensions") is not None else None,
                        "UpdateLogoAdExtensions": obj.get("UpdateLogoAdExtensions") if obj.get("UpdateLogoAdExtensions") is not None else None,
                        "UpdateLeadFormAdExtensions": obj.get("UpdateLeadFormAdExtensions") if obj.get("UpdateLeadFormAdExtensions") is not None else None,
                        "UpdateStatusForCampaigns": obj.get("UpdateStatusForCampaigns") if obj.get("UpdateStatusForCampaigns") is not None else None,
                        "UpdateStatusForAdGroups": obj.get("UpdateStatusForAdGroups") if obj.get("UpdateStatusForAdGroups") is not None else None,
                        "UpdateStatusForAds": obj.get("UpdateStatusForAds") if obj.get("UpdateStatusForAds") is not None else None,
                        "UpdateStatusForKeywords": obj.get("UpdateStatusForKeywords") if obj.get("UpdateStatusForKeywords") is not None else None,
                        "UpdateAdGroupNetwork": obj.get("UpdateAdGroupNetwork") if obj.get("UpdateAdGroupNetwork") is not None else None,
                        "UpdatePageFeeds": obj.get("UpdatePageFeeds") if obj.get("UpdatePageFeeds") is not None else None,
                        "UpdateAdCustomizerFeeds": obj.get("UpdateAdCustomizerFeeds") if obj.get("UpdateAdCustomizerFeeds") is not None else None,
                        "UpdateBrandSuitability": obj.get("UpdateBrandSuitability") if obj.get("UpdateBrandSuitability") is not None else None,
                        "UpdateAdCustomizerAttributes": obj.get("UpdateAdCustomizerAttributes") if obj.get("UpdateAdCustomizerAttributes") is not None else None,
                        "DeleteRemovedEntities": obj.get("DeleteRemovedEntities") if obj.get("DeleteRemovedEntities") is not None else None,
                        "RaiseBidsToMinimum": obj.get("RaiseBidsToMinimum") if obj.get("RaiseBidsToMinimum") is not None else None,
                        "RaiseCampaignBudgetsToMinimum": obj.get("RaiseCampaignBudgetsToMinimum") if obj.get("RaiseCampaignBudgetsToMinimum") is not None else None,
                        "AdjustmentForCampaignBudgets": obj.get("AdjustmentForCampaignBudgets") if obj.get("AdjustmentForCampaignBudgets") is not None else None,
                        "AdjustmentForBids": obj.get("AdjustmentForBids") if obj.get("AdjustmentForBids") is not None else None,
                        "SuffixForUrls": obj.get("SuffixForUrls") if obj.get("SuffixForUrls") is not None else None,
                        "SearchAndReplaceForUrls": ImportSearchAndReplaceForStringProperty.from_dict(obj["SearchAndReplaceForUrls"]) if obj.get("SearchAndReplaceForUrls") is not None else None,
                        "SuffixForTrackingTemplates": obj.get("SuffixForTrackingTemplates") if obj.get("SuffixForTrackingTemplates") is not None else None,
                        "SearchAndReplaceForTrackingTemplates": ImportSearchAndReplaceForStringProperty.from_dict(obj["SearchAndReplaceForTrackingTemplates"]) if obj.get("SearchAndReplaceForTrackingTemplates") is not None else None,
                        "SuffixForCampaignNames": obj.get("SuffixForCampaignNames") if obj.get("SuffixForCampaignNames") is not None else None,
                        "SearchAndReplaceForCampaignNames": ImportSearchAndReplaceForStringProperty.from_dict(obj["SearchAndReplaceForCampaignNames"]) if obj.get("SearchAndReplaceForCampaignNames") is not None else None,
                        "SearchAndReplaceForCustomParameters": ImportSearchAndReplaceForStringProperty.from_dict(obj["SearchAndReplaceForCustomParameters"]) if obj.get("SearchAndReplaceForCustomParameters") is not None else None,
                        "SearchAndReplaceForFinalURLSuffix": ImportSearchAndReplaceForStringProperty.from_dict(obj["SearchAndReplaceForFinalURLSuffix"]) if obj.get("SearchAndReplaceForFinalURLSuffix") is not None else None,
                        "PauseCampaignsWithoutSupportedLocations": obj.get("PauseCampaignsWithoutSupportedLocations") if obj.get("PauseCampaignsWithoutSupportedLocations") is not None else None,
                        "EnableParentLocationMapping": obj.get("EnableParentLocationMapping") if obj.get("EnableParentLocationMapping") is not None else None,
                        "AssociatedStoreId": obj.get("AssociatedStoreId") if obj.get("AssociatedStoreId") is not None else None,
                        "RaiseProductGroupBidsToMinimum": obj.get("RaiseProductGroupBidsToMinimum") if obj.get("RaiseProductGroupBidsToMinimum") is not None else None,
                        "SearchAndDsaMixedCampaignAsSearchCampaign": obj.get("SearchAndDsaMixedCampaignAsSearchCampaign") if obj.get("SearchAndDsaMixedCampaignAsSearchCampaign") is not None else None,
                        "AccountUrlOptions": obj.get("AccountUrlOptions") if obj.get("AccountUrlOptions") is not None else None,
                        "EnableAutoCurrencyConversion": obj.get("EnableAutoCurrencyConversion") if obj.get("EnableAutoCurrencyConversion") is not None else None,
                        "UpdateBiddingStrategies": obj.get("UpdateBiddingStrategies") if obj.get("UpdateBiddingStrategies") is not None else None,
                        "AssociatedUetTagId": obj.get("AssociatedUetTagId") if obj.get("AssociatedUetTagId") is not None else None,
                        "AutoDeviceBidOptimization": obj.get("AutoDeviceBidOptimization") if obj.get("AutoDeviceBidOptimization") is not None else None,
                        "AdScheduleUseSearcherTimezone": obj.get("AdScheduleUseSearcherTimezone") if obj.get("AdScheduleUseSearcherTimezone") is not None else None,
                        "PauseAIMAdGroupIfAllAudienceCriterionNotImported": obj.get("PauseAIMAdGroupIfAllAudienceCriterionNotImported") if obj.get("PauseAIMAdGroupIfAllAudienceCriterionNotImported") is not None else None,
                        "RenameCampaignNameWithSuffix": obj.get("RenameCampaignNameWithSuffix") if obj.get("RenameCampaignNameWithSuffix") is not None else None,
                        "UpdateAdUrls": obj.get("UpdateAdUrls") if obj.get("UpdateAdUrls") is not None else None,
                        "UpdateSitelinkUrls": obj.get("UpdateSitelinkUrls") if obj.get("UpdateSitelinkUrls") is not None else None,
                        "ForwardCompatibilityMap": [KeyValuePairOfstringAndstring.from_dict(_item) for _item in obj["ForwardCompatibilityMap"]] if obj.get("ForwardCompatibilityMap") is not None else None,
                        "Type": obj.get("Type") if obj.get("Type") is not None else 'GoogleImportOption'
        })
        return _obj



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class PromotionOccasion(str, Enum):
    UNKNOWN = 'Unknown'

    NEWYEARS = 'NewYears'

    VALENTINESDAY = 'ValentinesDay'

    EASTER = 'Easter'

    MOTHERSDAY = 'MothersDay'

    FATHERSDAY = 'FathersDay'

    LABORDAY = 'LaborDay'

    BACKTOSCHOOL = 'BackToSchool'

    HALLOWEEN = 'Halloween'

    BLACKFRIDAY = 'BlackFriday'

    CYBERMONDAY = 'CyberMonday'

    CHRISTMAS = 'Christmas'

    BOXINGDAY = 'BoxingDay'

    NONE = 'None'

    INDEPENDENCEDAY = 'IndependenceDay'

    NATIONALDAY = 'NationalDay'

    ENDOFSEASON = 'EndOfSeason'

    WINTERSALE = 'WinterSale'

    SUMMERSALE = 'SummerSale'

    FALLSALE = 'FallSale'

    SPRINGSALE = 'SpringSale'

    RAMADAN = 'Ramadan'

    EIDALFITR = 'EidAlFitr'

    EIDALADHA = 'EidAlAdha'

    SINGLESDAY = 'SinglesDay'

    WOMENSDAY = 'WomensDay'

    HOLI = 'Holi'

    PARENTSDAY = 'ParentsDay'

    STNICHOLASDAY = 'StNicholasDay'

    CHINESENEWYEAR = 'ChineseNewYear'

    CARNIVAL = 'Carnival'

    EPIPHANY = 'Epiphany'

    ROSHHASHANAH = 'RoshHashanah'

    PASSOVER = 'Passover'

    HANUKKAH = 'Hanukkah'

    DIWALI = 'Diwali'

    NAVRATRI = 'Navratri'

    SONGKRAN = 'Songkran'

    YEARENDGIFT = 'YearEndGift'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of PromotionOccasion from a JSON string"""
        return cls(json.loads(json_str))





from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *

class BulkDataExclusion(_SingleRecordBulkEntity):
    """ Represents an data exclusion.

    This class exposes the property :attr:`data_exclusion` that can be read and written as fields of the data exclusion record
    in a bulk file.

    For more information, see data exclusion at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, data_exclusion=None):
        super(BulkDataExclusion, self).__init__()

        self._data_exclusion = data_exclusion

    @property
    def data_exclusion(self):
        """ The DataExclusion Data Object of the Campaign Management Service.

        A subset of DataExclusion properties are available in the Data Exclusion record.
        For more information, see Data Exclusion at https://go.microsoft.com/fwlink/?linkid=846127.
        """
        return self._data_exclusion

    @data_exclusion.setter
    def data_exclusion(self, data_exclusion):
        self._data_exclusion = data_exclusion

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.data_exclusion.Id),
            csv_to_field=lambda c, v: setattr(c.data_exclusion, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.DataExclusion,
            field_to_csv=lambda c: bulk_str(c.data_exclusion.Name),
            csv_to_field=lambda c, v: setattr(c.data_exclusion, 'Name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.StartDate,
            field_to_csv=lambda c: bulk_datetime_str2(c.data_exclusion.StartDate),
            csv_to_field=lambda c, v: setattr(c.data_exclusion, 'StartDate', parse_datetime2(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.EndDate,
            field_to_csv=lambda c: bulk_datetime_str2(c.data_exclusion.EndDate),
            csv_to_field=lambda c, v: setattr(c.data_exclusion, 'EndDate', parse_datetime2(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.Description,
            field_to_csv=lambda c: c.data_exclusion.Description,
            csv_to_field=lambda c, v: setattr(c.data_exclusion, 'Description', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CampaignType,
            field_to_csv=lambda c: field_to_csv_CampaignType(c.data_exclusion),
            csv_to_field=lambda c, v: csv_to_field_CampaignType(c.data_exclusion, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.DeviceType,
            field_to_csv=lambda c: field_to_csv_DeviceType(c.data_exclusion),
            csv_to_field=lambda c, v: csv_to_field_DeviceType(c.data_exclusion, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CampaignAssociations,
            field_to_csv=lambda c: field_to_csv_CampaignAssociations(c.data_exclusion),
            csv_to_field=lambda c, v: csv_to_field_CampaignAssociations(c.data_exclusion, v)
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        self.data_exclusion = _CAMPAIGN_OBJECT_FACTORY_V13.create('DataExclusion')

        row_values.convert_to_entity(self, BulkDataExclusion._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._data_exclusion, 'DataExclusion')
        self.convert_to_values(row_values, BulkDataExclusion._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkDataExclusion, self).read_additional_data(stream_reader)



from os import path


class FileUploadParameters:
    """ Describes the available parameters when submitting a file for upload, such as the path of the upload result file. """

    def __init__(self,
                 upload_file_path,
                 result_file_directory=None,
                 result_file_name=None,
                 overwrite_result_file=False,
                 compress_upload_file=True,
                 response_mode='ErrorsAndResults',
                 timeout_in_milliseconds=None, 
                 rename_upload_file_to_match_request_id = True):
        """ Initialize a new instance of this class.

        :param result_file_directory: The directory where the file will be downloaded.
        :type result_file_directory: str
        :param result_file_name: The name of the download result file.
        :type result_file_name: str
        :param overwrite_result_file: (optional) Whether the local result file should be overwritten if it already exists, default is False.
        :type overwrite_result_file: bool
        :param upload_file_path: The fully qualified local path of the upload file.
        :type upload_file_path: str
        :param compress_upload_file: (optional) Determines whether the upload file should be compressed before uploading. The default value is True.
        :type compress_upload_file: bool
        :param response_mode: (optional) Determines whether the bulk service should return upload errors with the corresponding entity data.
                                If not specified, this property is set by default to ErrorsAndResults.
        :type response_mode: str
        :param timeout_in_milliseconds: (optional) timeout for bulk upload operations in milliseconds
        :type timeout_in_milliseconds: int
        """

        self._result_file_directory = result_file_directory
        self._result_file_name = result_file_name
        self._decompress_result_file = True
        if result_file_name is not None:
            _, ext = path.splitext(result_file_name)
            if ext == '.zip':
                self._decompress_result_file = False
        self._overwrite_result_file = overwrite_result_file
        self._submit_upload_parameters = SubmitUploadParameters(
            upload_file_path=upload_file_path,
            compress_upload_file=compress_upload_file,
            response_mode=response_mode,
            rename_upload_file_to_match_request_id= rename_upload_file_to_match_request_id
        )
        self._timeout_in_milliseconds=timeout_in_milliseconds

    @property
    def decompress_result_file(self):
        return self._decompress_result_file

    @property
    def upload_file_path(self):
        """ The fully qualified local path of the upload file.

        :rtype: str
        """

        return self._submit_upload_parameters.upload_file_path

    @upload_file_path.setter
    def upload_file_path(self, upload_file_path):
        self._submit_upload_parameters.upload_file_path = upload_file_path

    @property
    def compress_upload_file(self):
        """ Determines whether the upload file should be compressed before uploading. The default value is True.

        :rtype: bool
        """

        return self._submit_upload_parameters.compress_upload_file

    @compress_upload_file.setter
    def compress_upload_file(self, compress):
        self._submit_upload_parameters.compress_upload_file = compress

    @property
    def response_mode(self):
        """ Determines whether the bulk service should return upload errors with the corresponding entity data.

        If not specified, this property is set by default to ErrorsAndResults.

        :rtype: str
        """

        return self._submit_upload_parameters.response_mode

    @response_mode.setter
    def response_mode(self, value):
        self._submit_upload_parameters.response_mode = value

    @property
    def result_file_directory(self):
        """ The directory where the file will be downloaded.

        :rtype: str
        """

        return self._result_file_directory

    @result_file_directory.setter
    def result_file_directory(self, result_file_directory):
        self._result_file_directory = result_file_directory

    @property
    def result_file_name(self):
        """ The name of the download result file.

        :rtype: str
        """

        return self._result_file_name

    @result_file_name.setter
    def result_file_name(self, result_file_name):
        self._result_file_name = result_file_name

    @property
    def overwrite_result_file(self):
        """ Whether the local result file should be overwritten if it already exists.

        :rtype: bool
        """

        return self._overwrite_result_file

    @overwrite_result_file.setter
    def overwrite_result_file(self, overwrite):
        self._overwrite_result_file = overwrite

    @property
    def timeout_in_milliseconds(self):
        return self._timeout_in_milliseconds


class SubmitUploadParameters(object):
    """ Describes the minimum available parameters when submitting a file for upload, such as the path of the upload file. """

    def __init__(self,
                 upload_file_path,
                 compress_upload_file=True,
                 response_mode='ErrorsAndResults',
                 timeout_in_milliseconds=None,
                 rename_upload_file_to_match_request_id=True):
        """ Initialize a new instance of this class.

        :param upload_file_path: The fully qualified local path of the upload file.
        :type upload_file_path: str
        :param compress_upload_file: (optional) Determines whether the upload file should be compressed before uploading. The default value is True.
        :type compress_upload_file: bool
        :param response_mode: (optional) Determines whether the bulk service should return upload errors with the corresponding entity data.
                                If not specified, this property is set by default to ErrorsAndResults.
        :type response_mode: str
        :param timeout_in_milliseconds: (optional) timeout for submit upload operations in milliseconds
        :type timeout_in_milliseconds: int
        """

        self._upload_file_path = upload_file_path
        self._compress_upload_file = compress_upload_file
        self._response_mode = response_mode
        self._timeout_in_milliseconds = timeout_in_milliseconds
        self._rename_upload_file_to_match_request_id=rename_upload_file_to_match_request_id

    @property
    def rename_upload_file_to_match_request_id(self):
        """ rename the upload file to request id or not.

        :rtype: boolean
        """
        return self._rename_upload_file_to_match_request_id;

    @property
    def upload_file_path(self):
        """ The fully qualified local path of the upload file.

        :rtype: str
        """

        return self._upload_file_path

    @property
    def compress_upload_file(self):
        """ Determines whether the upload file should be compressed before uploading. The default value is True.

        :rtype: bool
        """

        return self._compress_upload_file

    @upload_file_path.setter
    def upload_file_path(self, upload_file_path):
        self._upload_file_path = upload_file_path

    @compress_upload_file.setter
    def compress_upload_file(self, compress):
        self._compress_upload_file = compress

    @property
    def response_mode(self):
        """ Determines whether the bulk service should return upload errors with the corresponding entity data.

        If not specified, this property is set by default to ErrorsAndResults.

        :rtype: str
        """

        return self._response_mode

    @response_mode.setter
    def response_mode(self, value):
        self._response_mode = value

    @property
    def timeout_in_milliseconds(self):
        return self._timeout_in_milliseconds

    @timeout_in_milliseconds.setter
    def timeout_in_milliseconds(self, value):
        self._timeout_in_milliseconds = value


class EntityUploadParameters(object):
    """ Describes the available parameters when submitting entities for upload, such as the entities that you want to upload. """

    def __init__(self,
                 entities,
                 result_file_directory=None,
                 result_file_name=None,
                 overwrite_result_file=False,
                 response_mode='ErrorsAndResults',
                 timeout_in_milliseconds=None):
        """ Initializes a new instance of this class.

        :param entities: The list of bulk entities that you want to upload.
        :type entities: collections.Iterable[BulkEntity]
        :param result_file_directory: (optional) The directory where the file will be downloaded.
        :type result_file_directory: str
        :param result_file_name: (optional) The name of the download result file.
        :type result_file_name: str
        :param overwrite_result_file: (optional) Whether the local result file should be overwritten if it already exists.
        :type overwrite_result_file: bool
        :param response_mode: (optional) Determines whether the bulk service should return upload errors with the corresponding entity data.
                              If not specified, this property is set by default to ErrorsAndResults.
        :type response_mode: str
        :param timeout_in_milliseconds: (optional) timeout for entity upload operations in milliseconds
        :type timeout_in_milliseconds: int
        """

        self._result_file_directory = result_file_directory
        self._result_file_name = result_file_name
        self._decompress_result_file = True
        if result_file_name is not None:
            _, ext = path.splitext(result_file_name)
            if ext == '.zip':
                self._decompress_result_file = False
        self._entities = entities
        self._overwrite_result_file = overwrite_result_file
        self._response_mode = response_mode
        self._timeout_in_milliseconds = timeout_in_milliseconds

    @property
    def decompress_result_file(self):
        return self._decompress_result_file

    @property
    def entities(self):
        """ The list of bulk entities that you want to upload.

        :rtype: collections.Iterable[BulkEntity]
        """

        return self._entities

    @entities.setter
    def entities(self, value):
        self._entities = value

    @property
    def response_mode(self):
        """ Determines whether the bulk service should return upload errors with the corresponding entity data.

        If not specified, this property is set by default to ErrorsAndResults.

        :rtype: str
        """

        return self._response_mode

    @response_mode.setter
    def response_mode(self, value):
        self._response_mode = value

    @property
    def result_file_directory(self):
        """ The directory where the file will be downloaded.

        :rtype: str
        """

        return self._result_file_directory

    @result_file_directory.setter
    def result_file_directory(self, result_file_directory):
        self._result_file_directory = result_file_directory

    @property
    def result_file_name(self):
        """ The name of the download result file.

        :rtype: str
        """

        return self._result_file_name

    @result_file_name.setter
    def result_file_name(self, result_file_name):
        self._result_file_name = result_file_name

    @property
    def overwrite_result_file(self):
        """ Whether the local result file should be overwritten if it already exists.

        :rtype: bool
        """

        return self._overwrite_result_file

    @overwrite_result_file.setter
    def overwrite_result_file(self, overwrite):
        self._overwrite_result_file = overwrite

    @property
    def timeout_in_milliseconds(self):
        return self._timeout_in_milliseconds

    @timeout_in_milliseconds.setter
    def timeout_in_milliseconds(self, value):
        self._timeout_in_milliseconds = value



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping, _DynamicColumnNameMapping
from bingads.v13.internal.extensions import *


class _BulkNegativeKeyword(_SingleRecordBulkEntity):
    """ The base class for all bulk negative keywords.

    Either assigned individually to a campaign or ad group entity, or shared in a negative keyword list.

    *See also:*

    * :class:`.BulkAdGroupNegativeKeyword`
    * :class:`.BulkCampaignNegativeKeyword`
    * :class:`.BulkSharedNegativeKeyword`
    """

    def __init__(self, status=None, negative_keyword=None, parent_id=None):
        super(_BulkNegativeKeyword, self).__init__()

        self._negative_keyword = negative_keyword
        self._status = status
        self._parent_id = parent_id

    @property
    def status(self):
        """ The status of the negative keyword association.

        The value is 'Active' if the negative keyword is assigned to the parent entity.
        The value is 'Deleted' if the negative keyword is removed from the parent entity,
                                    or should be removed in a subsequent upload operation.
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        self._status = status

    @property
    def negative_keyword(self):
        """ Defines a negative keyword with match type. """

        return self._negative_keyword

    @negative_keyword.setter
    def negative_keyword(self, negative_keyword):
        self._negative_keyword = negative_keyword

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.negative_keyword.Id),
            csv_to_field=lambda c, v: setattr(c.negative_keyword, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.status),
            csv_to_field=lambda c, v: setattr(c, '_status', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c._parent_id),
            csv_to_field=lambda c, v: setattr(c, '_parent_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Keyword,
            field_to_csv=lambda c: c.negative_keyword.Text,
            csv_to_field=lambda c, v: setattr(c.negative_keyword, 'Text', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.MatchType,
            field_to_csv=lambda c: bulk_str(c.negative_keyword.MatchType),
            csv_to_field=lambda c, v: csv_to_field_enum(c.negative_keyword, v, 'MatchType', MatchType)
        )
    ]

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._negative_keyword, 'negative_keyword')
        self.convert_to_values(row_values, _BulkNegativeKeyword._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        self._negative_keyword = _CAMPAIGN_OBJECT_FACTORY_V13.create('NegativeKeyword')
        self._negative_keyword.Type = 'NegativeKeyword'
        row_values.convert_to_entity(self, _BulkNegativeKeyword._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(_BulkNegativeKeyword, self).read_additional_data(stream_reader)


class _BulkEntityNegativeKeyword(_BulkNegativeKeyword):
    """ This base class for all bulk negative keywords that are assigned individually to a campaign or ad group entity.

    *See also:*

    * :class:`.BulkAdGroupNegativeKeyword`
    * :class:`.BulkCampaignNegativeKeyword`
    """

    def __init__(self,
                 status=None,
                 negative_keyword=None,
                 parent_id=None,
                 entity_name=None):
        super(_BulkEntityNegativeKeyword, self).__init__(
            status,
            negative_keyword,
            parent_id,
        )
        self._entity_name = entity_name

    @property
    def _entity_column_name(self):
        raise NotImplementedError()

    _MAPPINGS = [
        _DynamicColumnNameMapping(
            header_func=lambda c: c._entity_column_name,
            field_to_csv=lambda c: c._entity_name,
            csv_to_field=lambda c, v: setattr(c, '_entity_name', v))
    ]

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        super(_BulkEntityNegativeKeyword, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, _BulkEntityNegativeKeyword._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        super(_BulkEntityNegativeKeyword, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, _BulkEntityNegativeKeyword._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(_BulkEntityNegativeKeyword, self).read_additional_data(stream_reader)


class BulkAdGroupNegativeKeyword(_BulkEntityNegativeKeyword):
    """ Represents a negative keyword that is assigned to a ad group. Each negative keyword can be read or written in a bulk file.

    This class exposes the :attr:`.BulkNegativeKeyword.negative_keyword` property that can be read and written as
    fields of the Ad Group Negative Keyword record in a bulk file.

    For more information, see Ad Group Negative Keyword at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 status=None,
                 negative_keyword=None,
                 ad_group_id=None,
                 ad_group_name=None,
                 campaign_name=None):
        super(BulkAdGroupNegativeKeyword, self).__init__(
            status,
            negative_keyword,
            ad_group_id,
            ad_group_name,
        )
        self._campaign_name = campaign_name

    @property
    def campaign_name(self):
        """ The name of the campaign that the negative keyword is assigned.

        Corresponds to the 'Campaign' field in the bulk file.

        :rtype: str
        """

        return self._campaign_name

    @campaign_name.setter
    def campaign_name(self, campaign_name):
        self._campaign_name = campaign_name

    @property
    def ad_group_id(self):
        """ Corresponds to the 'Parent Id' field in the bulk file.

        :return: The identifier of the ad group that the negative keyword is assigned.
        :rtype: int
        """

        return self._parent_id

    @ad_group_id.setter
    def ad_group_id(self, value):
        self._parent_id = value

    @property
    def ad_group_name(self):
        """ Corresponds to the 'Ad Group' field in the bulk file.

        :return: The name of the ad group that the negative keyword is assigned.
        :rtype: str
        """

        return self._entity_name

    @ad_group_name.setter
    def ad_group_name(self, value):
        self._entity_name = value

    @property
    def _entity_column_name(self):
        return _StringTable.AdGroup

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Campaign,
            field_to_csv=lambda c: c.campaign_name,
            csv_to_field=lambda c, v: setattr(c, 'campaign_name', v)
        )
    ]

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        super(BulkAdGroupNegativeKeyword, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkAdGroupNegativeKeyword._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        super(BulkAdGroupNegativeKeyword, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkAdGroupNegativeKeyword._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkAdGroupNegativeKeyword, self).read_additional_data(stream_reader)


class BulkCampaignNegativeKeyword(_BulkEntityNegativeKeyword):
    """ Represents a negative keyword that is assigned to a campaign. Each negative keyword can be read or written in a bulk file.

    This class exposes the :attr:`BulkNegativeKeyword.negative_keyword` property that can be read and written as
    fields of the Campaign Negative Keyword record in a bulk file.

    For more information, see Campaign Negative Keyword at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 status=None,
                 negative_keyword=None,
                 campaign_id=None,
                 campaign_name=None):
        super(BulkCampaignNegativeKeyword, self).__init__(
            status,
            negative_keyword,
            campaign_id,
            campaign_name,
        )

    @property
    def campaign_id(self):
        """ The identifier of the campaign that the negative keyword is assigned.

        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: int
        """

        return self._parent_id

    @campaign_id.setter
    def campaign_id(self, value):
        self._parent_id = value

    @property
    def campaign_name(self):
        """ The name of the campaign that the negative keyword is assigned.

        Corresponds to the 'Campaign' field in the bulk file.

        :rtype: str
        """

        return self._entity_name

    @campaign_name.setter
    def campaign_name(self, value):
        self._entity_name = value

    @property
    def _entity_column_name(self):
        return _StringTable.Campaign


class BulkCampaignNegativeKeywordList(_SingleRecordBulkEntity):
    """ Represents a negative keyword list that is assigned to a campaign. Each negative keyword list can be read or written in a bulk file.

    This class exposes the :attr:`BulkCampaignNegativeKeywordList.SharedEntityAssociation` property that can be read
    and written as fields of the Campaign Negative Keyword List Association record in a bulk file.

    For more information, see Campaign Negative Keyword List Association at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, status=None, shared_entity_association=None):
        super(BulkCampaignNegativeKeywordList, self).__init__()

        self._shared_entity_association = shared_entity_association
        self._status = status

    @property
    def status(self):
        """ The status of the negative keyword list association.

        :rtype: str
        """

        return self._status

    @status.setter
    def status(self, status):
        self._status = status

    @property
    def shared_entity_association(self):
        """ The campaign and negative keyword list identifiers.

        see Campaign Negative Keyword List Association at https://go.microsoft.com/fwlink/?linkid=846127.
        """

        return self._shared_entity_association

    @shared_entity_association.setter
    def shared_entity_association(self, shared_entity_association):
        self._shared_entity_association = shared_entity_association

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: c.status,
            csv_to_field=lambda c, v: setattr(c, 'status', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.shared_entity_association.SharedEntityId),
            csv_to_field=lambda c, v: setattr(c.shared_entity_association, 'SharedEntityId', int(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.shared_entity_association.EntityId),
            csv_to_field=lambda c, v: setattr(c.shared_entity_association, 'EntityId', int(v))
        ),
    ]

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._shared_entity_association, 'shared_entity_association')
        self.convert_to_values(row_values, BulkCampaignNegativeKeywordList._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        self._shared_entity_association = _CAMPAIGN_OBJECT_FACTORY_V13.create('SharedEntityAssociation')
        self._shared_entity_association.EntityType = 'Campaign'
        self._shared_entity_association.SharedEntityType = 'NegativeKeywordList'
        row_values.convert_to_entity(self, BulkCampaignNegativeKeywordList._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkCampaignNegativeKeywordList, self).read_additional_data(stream_reader)

class BulkNegativeKeywordList(_SingleRecordBulkEntity):
    """ Represents a negative keyword list that can be read or written in a bulk file.

    This class exposes the :attr:`.BulkNegativeKeywordList.negative_keyword_list` property that can be read and
    written as fields of the Negative Keyword List record in a bulk file.

    For more information, see Negative Keyword List at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, status=None, negative_keyword_list=None):
        super(BulkNegativeKeywordList, self).__init__()

        self._status = status
        self._negative_keyword_list = negative_keyword_list

    @property
    def negative_keyword_list(self):
        """ The negative keyword list.

        see Negative Keyword List at https://go.microsoft.com/fwlink/?linkid=846127.
        """

        return self._negative_keyword_list

    @negative_keyword_list.setter
    def negative_keyword_list(self, negative_keyword_list):
        self._negative_keyword_list = negative_keyword_list

    @property
    def status(self):
        """ The status of the negative keyword list.

        :rtype: str
        """

        return self._status

    @status.setter
    def status(self, status):
        self._status = status

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.negative_keyword_list.Id),
            csv_to_field=lambda c, v: setattr(c.negative_keyword_list, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.status),
            csv_to_field=lambda c, v: setattr(c, 'status', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Name,
            field_to_csv=lambda c: c.negative_keyword_list.Name,
            csv_to_field=lambda c, v: setattr(c.negative_keyword_list, 'Name', v)
        )
    ]

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._negative_keyword_list, 'negative_keyword_list')
        self.convert_to_values(row_values, BulkNegativeKeywordList._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        self._negative_keyword_list = _CAMPAIGN_OBJECT_FACTORY_V13.create('NegativeKeywordList')
        self._negative_keyword_list.Type = 'NegativeKeywordList'
        row_values.convert_to_entity(self, BulkNegativeKeywordList._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkNegativeKeywordList, self).read_additional_data(stream_reader)


class BulkSharedNegativeKeyword(_BulkNegativeKeyword):
    """ Represents a negative keyword that is shared in a negative keyword list.

    Each shared negative keyword can be read or written in a bulk file.
    This class exposes the :attr:`.BulkNegativeKeyword.NegativeKeyword` property that
    can be read and written as fields of the Shared Negative Keyword record in a bulk file.

    For more information, see Shared Negative Keyword at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, status=None, negative_keyword=None, negative_keyword_list_id=None):
        super(BulkSharedNegativeKeyword, self).__init__(status, negative_keyword, negative_keyword_list_id)

    @property
    def negative_keyword_list_id(self):
        return self._parent_id

    @negative_keyword_list_id.setter
    def negative_keyword_list_id(self, value):
        self._parent_id = value

class BulkAccountSharedNegativeKeyword(_BulkNegativeKeyword):
    """ Represents an account negative keyword that is shared in a negative keyword list.

    Each shared negative keyword can be read or written in a bulk file.
    This class exposes the :attr:`.BulkNegativeKeyword.NegativeKeyword` property that
    can be read and written as fields of the Account Shared Negative Keyword record in a bulk file.

    For more information, see Account Shared Negative Keyword at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, status=None, negative_keyword=None, negative_keyword_list_id=None):
        super(BulkAccountSharedNegativeKeyword, self).__init__(status, negative_keyword, negative_keyword_list_id)

    @property
    def negative_keyword_list_id(self):
        return self._parent_id

    @negative_keyword_list_id.setter
    def negative_keyword_list_id(self, value):
        self._parent_id = value



# coding: utf-8

"""
    BulkService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.bulk.fault_source import FaultSource
from typing_extensions import Self

class AdCenterInternalErrorFault(BaseModel):
    """
    AdCenterInternalErrorFault
    """ # noqa: E501
    error_message: Optional[StrictStr] = Field(default=None, alias="ErrorMessage")
    mt_server_name: Optional[StrictStr] = Field(default=None, alias="MTServerName")
    faulting_layer: Optional[FaultSource] = Field(default=None, alias="FaultingLayer")
    error_source_system: Optional[StrictStr] = Field(default=None, alias="ErrorSourceSystem")
    tracking_id: Optional[StrictStr] = Field(default=None, alias="TrackingId")
    type: Optional[StrictStr] = Field(default='AdCenterInternalErrorFault', alias="Type")
    __properties: ClassVar[List[str]] = ["ErrorMessage", "MTServerName", "FaultingLayer", "ErrorSourceSystem", "TrackingId", "Type"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	
    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())
	
    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of AdCenterInternalErrorFault from a JSON string"""
        return cls.from_dict(json.loads(json_str))
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if error_message (nullable) is None
        # and model_fields_set contains the field
        if self.error_message is None and "error_message" in self.model_fields_set:
            _dict['ErrorMessage'] = None

        # set to None if mt_server_name (nullable) is None
        # and model_fields_set contains the field
        if self.mt_server_name is None and "mt_server_name" in self.model_fields_set:
            _dict['MTServerName'] = None

        # set to None if faulting_layer (nullable) is None
        # and model_fields_set contains the field
        if self.faulting_layer is None and "faulting_layer" in self.model_fields_set:
            _dict['FaultingLayer'] = None

        # set to None if error_source_system (nullable) is None
        # and model_fields_set contains the field
        if self.error_source_system is None and "error_source_system" in self.model_fields_set:
            _dict['ErrorSourceSystem'] = None

        # set to None if tracking_id (nullable) is None
        # and model_fields_set contains the field
        if self.tracking_id is None and "tracking_id" in self.model_fields_set:
            _dict['TrackingId'] = None

        # set to None if type (nullable) is None
        # and model_fields_set contains the field
        if self.type is None and "type" in self.model_fields_set:
            _dict['Type'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AdCenterInternalErrorFault from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "ErrorMessage": obj.get("ErrorMessage") if obj.get("ErrorMessage") is not None else None,
                        "MTServerName": obj.get("MTServerName") if obj.get("MTServerName") is not None else None,
                        "FaultingLayer": obj.get("FaultingLayer") if obj.get("FaultingLayer") is not None else None,
                        "ErrorSourceSystem": obj.get("ErrorSourceSystem") if obj.get("ErrorSourceSystem") is not None else None,
                        "TrackingId": obj.get("TrackingId") if obj.get("TrackingId") is not None else None,
                        "Type": obj.get("Type") if obj.get("Type") is not None else 'AdCenterInternalErrorFault'
        })
        return _obj



# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.adinsight.day_month_and_year import DayMonthAndYear
from openapi_client.models.adinsight.entity_type import EntityType
from openapi_client.models.adinsight.kpi_type import KPIType
from openapi_client.models.adinsight.performance_insights_message import PerformanceInsightsMessage
from typing_extensions import Self

class PerformanceInsightsDetail(BaseModel):
    """
    PerformanceInsightsDetail
    """ # noqa: E501
    entity_id: Optional[StrictStr] = Field(default=None, alias="EntityId")
    entity_type: Optional[EntityType] = Field(default=None, alias="EntityType")
    kpi_type: Optional[KPIType] = Field(default=None, alias="KPIType")
    var_date: Optional[DayMonthAndYear] = Field(default=None, alias="Date")
    description: Optional[PerformanceInsightsMessage] = Field(default=None, alias="Description")
    root_causes: Optional[List[Optional[PerformanceInsightsMessage]]] = Field(default=None, alias="RootCauses")
    actions: Optional[List[Optional[PerformanceInsightsMessage]]] = Field(default=None, alias="Actions")
    __properties: ClassVar[List[str]] = ["EntityId", "EntityType", "KPIType", "Date", "Description", "RootCauses", "Actions"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of var_date
        if self.var_date:
            _dict['Date'] = self.var_date.to_dict()
        # override the default output from pydantic by calling `to_dict()` of description
        if self.description:
            _dict['Description'] = self.description.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in root_causes (list)
        _items = []
        if self.root_causes:
            for _item_root_causes in self.root_causes:
                if _item_root_causes:
                    _items.append(_item_root_causes.to_dict())
            _dict['RootCauses'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in actions (list)
        _items = []
        if self.actions:
            for _item_actions in self.actions:
                if _item_actions:
                    _items.append(_item_actions.to_dict())
            _dict['Actions'] = _items
        # set to None if entity_id (nullable) is None
        # and model_fields_set contains the field
        if self.entity_id is None and "entity_id" in self.model_fields_set:
            _dict['EntityId'] = None

        # set to None if entity_type (nullable) is None
        # and model_fields_set contains the field
        if self.entity_type is None and "entity_type" in self.model_fields_set:
            _dict['EntityType'] = None

        # set to None if kpi_type (nullable) is None
        # and model_fields_set contains the field
        if self.kpi_type is None and "kpi_type" in self.model_fields_set:
            _dict['KPIType'] = None

        # set to None if var_date (nullable) is None
        # and model_fields_set contains the field
        if self.var_date is None and "var_date" in self.model_fields_set:
            _dict['Date'] = None

        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict['Description'] = None

        # set to None if root_causes (nullable) is None
        # and model_fields_set contains the field
        if self.root_causes is None and "root_causes" in self.model_fields_set:
            _dict['RootCauses'] = None

        # set to None if actions (nullable) is None
        # and model_fields_set contains the field
        if self.actions is None and "actions" in self.model_fields_set:
            _dict['Actions'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PerformanceInsightsDetail from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "EntityId": obj.get("EntityId") if obj.get("EntityId") is not None else None,
                        "EntityType": obj.get("EntityType") if obj.get("EntityType") is not None else None,
                        "KPIType": obj.get("KPIType") if obj.get("KPIType") is not None else None,
                        "Date": DayMonthAndYear.from_dict(obj["Date"]) if obj.get("Date") is not None else None,
                        "Description": PerformanceInsightsMessage.from_dict(obj["Description"]) if obj.get("Description") is not None else None,
                        "RootCauses": [PerformanceInsightsMessage.from_dict(_item) for _item in obj["RootCauses"]] if obj.get("RootCauses") is not None else None,
                        "Actions": [PerformanceInsightsMessage.from_dict(_item) for _item in obj["Actions"]] if obj.get("Actions") is not None else None
        })
        return _obj



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from typing_extensions import Self

class VideoTemplateFilter(BaseModel):
    """
    VideoTemplateFilter
    """ # noqa: E501
    themes: Optional[List[StrictStr]] = Field(default=None, alias="Themes")
    aspect_ratios: Optional[List[StrictStr]] = Field(default=None, alias="AspectRatios")
    template_ids: Optional[List[StrictStr]] = Field(default=None, alias="TemplateIds")
    durations: Optional[List[StrictInt]] = Field(default=None, alias="Durations")
    max_media_asset_count: Optional[StrictInt] = Field(default=None, alias="MaxMediaAssetCount")
    max_text_asset_count: Optional[StrictInt] = Field(default=None, alias="MaxTextAssetCount")
    __properties: ClassVar[List[str]] = ["Themes", "AspectRatios", "TemplateIds", "Durations", "MaxMediaAssetCount", "MaxTextAssetCount"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if themes (nullable) is None
        # and model_fields_set contains the field
        if self.themes is None and "themes" in self.model_fields_set:
            _dict['Themes'] = None

        # set to None if aspect_ratios (nullable) is None
        # and model_fields_set contains the field
        if self.aspect_ratios is None and "aspect_ratios" in self.model_fields_set:
            _dict['AspectRatios'] = None

        # set to None if template_ids (nullable) is None
        # and model_fields_set contains the field
        if self.template_ids is None and "template_ids" in self.model_fields_set:
            _dict['TemplateIds'] = None

        # set to None if durations (nullable) is None
        # and model_fields_set contains the field
        if self.durations is None and "durations" in self.model_fields_set:
            _dict['Durations'] = None

        # set to None if max_media_asset_count (nullable) is None
        # and model_fields_set contains the field
        if self.max_media_asset_count is None and "max_media_asset_count" in self.model_fields_set:
            _dict['MaxMediaAssetCount'] = None

        # set to None if max_text_asset_count (nullable) is None
        # and model_fields_set contains the field
        if self.max_text_asset_count is None and "max_text_asset_count" in self.model_fields_set:
            _dict['MaxTextAssetCount'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of VideoTemplateFilter from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Themes": obj.get("Themes"),
                        "AspectRatios": obj.get("AspectRatios"),
                        "TemplateIds": obj.get("TemplateIds"),
                        "Durations": obj.get("Durations"),
                        "MaxMediaAssetCount": obj.get("MaxMediaAssetCount") if obj.get("MaxMediaAssetCount") is not None else None,
                        "MaxTextAssetCount": obj.get("MaxTextAssetCount") if obj.get("MaxTextAssetCount") is not None else None
        })
        return _obj



from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.bulk_object import _BulkObject
from bingads.v13.internal.extensions import *

from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13


class _ProductConditionHelper:
    def __init__(self):
        pass

    MAX_NUMBER_OF_CONDITIONS = 8

    @staticmethod
    def add_conditions_from_row_values(row_values, conditions):
        """

        :param row_values:
        :type row_values: _RowValues
        :param conditions:
        :type conditions: list[ProductCondition]
        :rtype: None
        """

        condition_header_prefix = _StringTable.ProductCondition1[:-1]
        value_header_prefix = _StringTable.ProductValue1[:-1]

        for i in range(1, _ProductConditionHelper.MAX_NUMBER_OF_CONDITIONS + 1):
            condition_success, product_condition = row_values.try_get_value(condition_header_prefix + str(i))
            value_success, product_value = row_values.try_get_value(value_header_prefix + str(i))

            if product_condition and product_value:
                condition = _CAMPAIGN_OBJECT_FACTORY_V13.create('ProductCondition')
                condition.Operand = product_condition
                condition.Attribute = product_value
                conditions.append(condition)

    @staticmethod
    def add_row_values_from_conditions(conditions, row_values):
        """

        :param conditions:
        :type conditions: list[ProductCondition]
        :param row_values:
        :type row_values: _RowValues
        :rtype: None
        """

        condition_header_prefix = _StringTable.ProductCondition1[:-1]
        value_header_prefix = _StringTable.ProductValue1[:-1]

        for i in range(1, len(conditions) + 1):
            row_values[condition_header_prefix + str(i)] = conditions[i - 1].Operand
            row_values[value_header_prefix + str(i)] = conditions[i - 1].Attribute



class QualityScoreData(_BulkObject):
    """ Represents a subset of the fields available in bulk records that support quality score data.

    For example :class:`.BulkKeyword`. For more information, see Bulk File Schema at https://go.microsoft.com/fwlink/?linkid=846127.
    """

    def __init__(self):
        self._quality_score = None
        self._keyword_relevance = None
        self._landing_page_relevance = None
        self._landing_page_user_experience = None

    @property
    def quality_score(self):
        """ Corresponds to the 'Quality Score' field in the bulk file.

        :rtype: int
        """

        return self._quality_score

    @property
    def keyword_relevance(self):
        """ Corresponds to the 'Keyword Relevance' field in the bulk file.

        :rtype: int
        """

        return self._keyword_relevance

    @property
    def landing_page_relevance(self):
        """ Corresponds to the 'Landing Page Relevance' field in the bulk file.

        :rtype: int
        """

        return self._landing_page_relevance

    @property
    def landing_page_user_experience(self):
        """ Corresponds to the 'Landing Page User Experience' field in the bulk file.

        :rtype: int
        """

        return self._landing_page_user_experience

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.QualityScore,
            field_to_csv=lambda c: bulk_str(c.quality_score),
            csv_to_field=lambda c, v: setattr(c, '_quality_score', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.KeywordRelevance,
            field_to_csv=lambda c: bulk_str(c.keyword_relevance),
            csv_to_field=lambda c, v: setattr(c, '_keyword_relevance', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.LandingPageRelevance,
            field_to_csv=lambda c: bulk_str(c.landing_page_relevance),
            csv_to_field=lambda c, v: setattr(c, '_landing_page_relevance', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.LandingPageUserExperience,
            field_to_csv=lambda c: bulk_str(c.landing_page_user_experience),
            csv_to_field=lambda c, v: setattr(c, '_landing_page_user_experience', int(v) if v else None)
        ),
    ]

    @staticmethod
    def read_from_row_values_or_null(row_values):
        quality_score_data = QualityScoreData()
        quality_score_data.read_from_row_values(row_values)
        return quality_score_data if quality_score_data.has_any_values else None

    @staticmethod
    def write_to_row_values_if_not_null(quality_score_data, row_values):
        if quality_score_data is not None:
            quality_score_data.write_to_row_values(row_values, False)

    def read_from_row_values(self, row_values):
        row_values.convert_to_entity(self, QualityScoreData._MAPPINGS)

    def write_to_row_values(self, row_values, exclude_readonly_data):
        self.convert_to_values(row_values, QualityScoreData._MAPPINGS)

    @property
    def has_any_values(self):
        return \
            self.quality_score \
            or self.keyword_relevance \
            or self.landing_page_relevance \
            or self.landing_page_user_experience

    def write_to_stream(self, row_writer, exclude_readonly_data):
        pass

    def read_related_data_from_stream(self, stream_reader):
        super(QualityScoreData, self).read_related_data_from_stream(stream_reader)

    def enclose_in_multiline_entity(self):
        pass

    @property
    def can_enclose_in_multiline_entity(self):
        return super(QualityScoreData, self).can_enclose_in_multiline_entity()



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping, _ComplexBulkMapping
from bingads.v13.internal.extensions import *

class BulkAudienceGroup(_SingleRecordBulkEntity):
    """ Represents an audience group.

    This class exposes the property :attr:`audience_group` that can be read and written as fields of the Audience Group record
    in a bulk file.

    For more information, see Audience Group at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, account_id=None, status=None, audience_group=None, audience_ids=None):
        super(BulkAudienceGroup, self).__init__()

        self._account_id = account_id
        self._audience_group = audience_group
        self._status = status
        self._audience_ids = audience_ids
        self._age_ranges = None
        self._gender_types = None

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, status):
        self._status = status

    @property
    def audience_ids(self):
        return self._audience_ids

    @audience_ids.setter
    def audience_ids(self, audience_ids):
        self._audience_ids = audience_ids

    @property
    def age_ranges(self):
        return self._age_ranges

    @age_ranges.setter
    def age_ranges(self, age_ranges):
        self._age_ranges = age_ranges

    @property
    def gender_types(self):
        return self._gender_types

    @gender_types.setter
    def gender_types(self, gender_types):
        self._gender_types = gender_types

    @property
    def account_id(self):
        """ The identifier of the account that contains the audience group.

        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: int
        """

        return self._account_id

    @account_id.setter
    def account_id(self, account_id):
        self._account_id = account_id

    @property
    def audience_group(self):
        """ The AudienceGroup Data Object of the Campaign Management Service.

        A subset of AudienceGroup properties are available in the Audience Group record.
        For more information, see Ad Group at https://go.microsoft.com/fwlink/?linkid=846127.
        """
        return self._audience_group

    @audience_group.setter
    def audience_group(self, audience_group):
        self._audience_group = audience_group

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.audience_group.Id),
            csv_to_field=lambda c, v: setattr(c.audience_group, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.status),
            csv_to_field=lambda c, v: setattr(c, 'status', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.account_id),
            csv_to_field=lambda c, v: setattr(c, 'account_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AudienceGroupName,
            field_to_csv=lambda c: c.audience_group.Name,
            csv_to_field=lambda c, v: setattr(c.audience_group, 'Name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Audiences,
            field_to_csv=lambda c: field_to_csv_AudienceIds(c),
            csv_to_field=lambda c, v: csv_to_field_AudienceIds(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AgeRanges,
            field_to_csv=lambda c: field_to_csv_AgeRanges(c),
            csv_to_field=lambda c, v: csv_to_field_AgeRanges(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.GenderTypes,
            field_to_csv=lambda c: field_to_csv_GenderTypes(c),
            csv_to_field=lambda c, v: csv_to_field_GenderTypes(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Description,
            field_to_csv=lambda c: bulk_str(c.audience_group.Description),
            csv_to_field=lambda c, v: setattr(c.audience_group, 'Description', v)
        ),
    ]


    def process_mappings_from_row_values(self, row_values):
        self.audience_group = _CAMPAIGN_OBJECT_FACTORY_V13.create('AudienceGroup')

        row_values.convert_to_entity(self, BulkAudienceGroup._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._audience_group, 'AudienceGroup')
        self.convert_to_values(row_values, BulkAudienceGroup._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkAudienceGroup, self).read_additional_data(stream_reader)



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *
from datetime import datetime

class BulkFeedItem(_SingleRecordBulkEntity):
    """ Represents a feed item.

    Properties of this class and of classes that it is derived from, correspond to fields of the Feed Item record in a bulk file.
    For more information, see Feed at https://go.microsoft.com/fwlink/?linkid=846127
    
    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """
    
    @staticmethod
    def _bulk_datetime_optional_str(value, id):
        if value is None:
            return None
        
        if not value:
            return "delete_value" if id > 0 else None
        
        return value.strftime('%Y/%m/%d %H:%M:%S')
    
    @staticmethod
    def _parse_datetime(value):
        
        if not value:
            return None
        try:
            if value == 'delete_value':
                return None
            else:
                return datetime.strptime(value, '%Y/%m/%d %H:%M:%S')
        except Exception:
            return parse_datetime(value)
        

    def __init__(self):
        super(BulkFeedItem, self).__init__()
        self._id = None
        self._feed_id = None
        self._campaign = None
        self._ad_group = None
        self._audience_id = None
        self._custom_attributes = None
        self._status = None
        self._start_date = None
        self._end_date = None
        self._keyword = None
        self._daytime_ranges = None
        self._match_type = None
        self._location_id = None
        self._intent_option = None
        self._device_preference = None
        self._adgroup_id = None
        self._campaign_id = None



    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.id),
            csv_to_field=lambda c, v: setattr(c, 'id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.feed_id),
            csv_to_field=lambda c, v: setattr(c, 'feed_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AudienceId,
            field_to_csv=lambda c: bulk_str(c.audience_id),
            csv_to_field=lambda c, v: setattr(c, 'audience_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Target,
            field_to_csv=lambda c: bulk_str(c.location_id),
            csv_to_field=lambda c, v: setattr(c, 'location_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: c.status,
            csv_to_field=lambda c, v: setattr(c, 'status', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.MatchType,
            field_to_csv=lambda c: c.match_type,
            csv_to_field=lambda c, v: setattr(c, 'match_type', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.PhysicalIntent,
            field_to_csv=lambda c: c.intent_option,
            csv_to_field=lambda c, v: setattr(c, 'intent_option', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Campaign,
            field_to_csv=lambda c: bulk_str(c.campaign),
            csv_to_field=lambda c, v: setattr(c, 'campaign', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AdGroup,
            field_to_csv=lambda c: bulk_str(c.ad_group),
            csv_to_field=lambda c, v: setattr(c, 'ad_group', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Keyword,
            field_to_csv=lambda c: bulk_str(c.keyword),
            csv_to_field=lambda c, v: setattr(c, 'keyword', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CustomAttributes,
            field_to_csv=lambda c: bulk_str(c.custom_attributes),
            csv_to_field=lambda c, v: setattr(c, 'custom_attributes', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.DevicePreference,
            field_to_csv=lambda c: bulk_device_preference_str(c.device_preference),
            csv_to_field=lambda c, v: setattr(c, 'device_preference', parse_device_preference(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.StartDate,
            field_to_csv=lambda c: BulkFeedItem._bulk_datetime_optional_str(c.start_date, c.id),
            csv_to_field=lambda c, v: setattr(c, 'start_date', BulkFeedItem._parse_datetime(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.EndDate,
            field_to_csv=lambda c: BulkFeedItem._bulk_datetime_optional_str(c.end_date, c.id),
            csv_to_field=lambda c, v: setattr(c, 'end_date', BulkFeedItem._parse_datetime(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.AdSchedule,
            field_to_csv=lambda c: field_to_csv_FeedItemAdSchedule(c, c.id),
            csv_to_field=lambda c, v: csv_to_field_FeedItemAdSchedule(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.TargetAdGroupId,
            field_to_csv=lambda c: bulk_optional_str(c.adgroup_id, c.id),
            csv_to_field=lambda c, v: setattr(c, 'adgroup_id', int(v) if v else '')
        ),
        _SimpleBulkMapping(
            header=_StringTable.TargetCampaignId,
            field_to_csv=lambda c: bulk_optional_str(c.campaign_id, c.id),
            csv_to_field=lambda c, v: setattr(c, 'campaign_id', int(v) if v else '')
        ),
    ]
    

    @property
    def device_preference(self):
        """ the id of the account which contains the feed
        Corresponds to the 'Device Preference' field in the bulk file.

        :rtype: long
        """
        return self._device_preference

    @device_preference.setter
    def device_preference(self, value):
        self._device_preference = value

    @property
    def intent_option(self):
        """ the id of the account which contains the feed
        Corresponds to the 'Physical Intent' field in the bulk file.

        :rtype: str
        """
        return self._intent_option

    @intent_option.setter
    def intent_option(self, value):
        self._intent_option = value
        
    @property
    def location_id(self):
        """ the id of the account which contains the feed
        Corresponds to the 'Target' field in the bulk file.

        :rtype: long
        """
        return self._location_id

    @location_id.setter
    def location_id(self, value):
        self._location_id = value
        
        
    @property
    def adgroup_id(self):
        """ the id of the account which contains the feed
        Corresponds to the 'Target Ad Group Id' field in the bulk file.

        :rtype: long
        """
        return self._adgroup_id

    @adgroup_id.setter
    def adgroup_id(self, value):
        self._adgroup_id = value

    @property
    def campaign_id(self):
        """ the id of the account which contains the feed
        Corresponds to the 'Target Campaign Id' field in the bulk file.

        :rtype: long
        """
        return self._campaign_id

    @campaign_id.setter
    def campaign_id(self, value):
        self._campaign_id = value

    @property
    def match_type(self):
        """ the match type of bulk record
        Corresponds to the 'Match Type' field in the bulk file.

        :rtype: str
        """
        return self._match_type

    @match_type.setter
    def match_type(self, value):
        self._match_type = value  
  
    @property
    def keyword(self):
        """ the status of bulk record
        Corresponds to the 'Keyword' field in the bulk file.

        :rtype: str
        """
        return self._keyword

    @keyword.setter
    def keyword(self, value):
        self._keyword = value
              
    @property
    def daytime_ranges(self):
        """ the status of bulk record
        Corresponds to the 'Ad Schedule' field in the bulk file.

        :rtype: str
        """
        return self._daytime_ranges

    @daytime_ranges.setter
    def daytime_ranges(self, value):
        self._daytime_ranges = value  
              
    @property
    def end_date(self):
        """ the status of bulk record
        Corresponds to the 'End Date' field in the bulk file.

        :rtype: str
        """
        return self._end_date

    @end_date.setter
    def end_date(self, value):
        self._end_date = value    
        
    @property
    def start_date(self):
        """ the status of bulk record
        Corresponds to the 'Start Date' field in the bulk file.

        :rtype: str
        """
        return self._start_date

    @start_date.setter
    def start_date(self, value):
        self._start_date = value


    @property
    def custom_attributes(self):
        """ the status of bulk record
        Corresponds to the 'Custom Attributes' field in the bulk file.

        :rtype: str
        """
        return self._custom_attributes

    @custom_attributes.setter
    def custom_attributes(self, value):
        self._custom_attributes = value

    @property
    def audience_id(self):
        """ the status of bulk record
        Corresponds to the 'Audience Id' field in the bulk file.

        :rtype: long
        """
        return self._audience_id

    @audience_id.setter
    def audience_id(self, value):
        self._audience_id = value

    @property
    def ad_group(self):
        """ the status of bulk record
        Corresponds to the 'Ad Group' field in the bulk file.

        :rtype: str
        """
        return self._ad_group

    @ad_group.setter
    def ad_group(self, value):
        self._ad_group = value

    @property
    def id(self):
        """ the status of bulk record
        Corresponds to the 'Id' field in the bulk file.

        :rtype: long
        """
        return self._id

    @id.setter
    def id(self, value):
        self._id = value
        

    @property
    def feed_id(self):
        """ the status of bulk record
        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: long
        """
        return self._feed_id

    @feed_id.setter
    def feed_id(self, value):
        self._feed_id = value    

    @property
    def status(self):
        """ the status of bulk record
        Corresponds to the 'Status' field in the bulk file.

        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, value):
        self._status = value

    @property
    def campaign(self):
        """ the id of the account which contains the feed
        Corresponds to the 'campaign' field in the bulk file.

        :rtype: str
        """
        return self._campaign

    @campaign.setter
    def campaign(self, value):
        self._campaign = value
 
    def process_mappings_from_row_values(self, row_values):
        row_values.convert_to_entity(self, BulkFeedItem._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self.convert_to_values(row_values, BulkFeedItem._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkFeedItem, self).read_additional_data(stream_reader)



from abc import ABCMeta, abstractmethod, abstractproperty

from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.bulk import EntityWriteException


class _BulkObject(metaclass=ABCMeta):
    """ The abstract base class for all bulk objects that can be read and written in a file that conforms to the Bing Ad Bulk File Schema.

    For more information about the Bulk File Schema, see https://go.microsoft.com/fwlink/?linkid=846127.
    """

    @abstractmethod
    def read_from_row_values(self, row_values):
        """ Read object data from a single row.

        *Example:*

        * SingleLineBulkEntity: reads entity fields.
        * BulkError: reads error fields.
        * BulkEntityIdentifier: reads identifier fields (Id, status etc.).

        :param row_values:
        :type row_values: _RowValues
        """

        raise NotImplementedError()

    @abstractmethod
    def write_to_row_values(self, row_values, exclude_readonly_data):
        """ Writes object data to a single row.

        *Example:*

        * SingleLineBulkEntity: writes entity fields.
        * BulkEntityIdentifier: writes identifier fields (Id, status etc.)

        :param row_values:
        :type row_values: _RowValues
        """

        raise NotImplementedError()

    @abstractmethod
    def read_related_data_from_stream(self, stream_reader):
        """ Reads object data from consecutive rows.

        *Example:*

        * SingleLineBulkEntity: reads entity errors.
        * MultilineBulkEntity: reads child entities.

        :param stream_reader:
        :type stream_reader: _BulkStreamReader
        """

        pass

    @abstractmethod
    def write_to_stream(self, row_writer, exclude_readonly_data):
        """ Writes object data to consecutive rows.

        *Example:*

        * SingleLineBulkEntity: writes entity.
        * MultilineBulkEntity: writes child entities.
        * BulkEntityIdentifier: writes identifier information (Id, status etc.)

        :param row_writer:
        :type row_writer: :class:`._BulkObjectWriter`
        """

        raise NotImplementedError()

    @abstractproperty
    def can_enclose_in_multiline_entity(self):
        """ Returns true if the entity is part of multiline entity, false otherwise.

        *Example:*

        * BulkSiteLinkAdExtension: returns true
        * BulkCampaignTarget: returns true
        * BulkAdGroup: returns false
        * BulkKeyword: returns false

        :rtype: bool
        """
        return False

    @abstractmethod
    def enclose_in_multiline_entity(self):
        """ Creates a multiline entity containing this entity

        *Example:*

        * BulkSiteLink: returns BulkSiteLinkAdExtension containing this BulkSiteLink
        * BulkCampaignAgeTargetBid: return BulkCampaignTarget containing this BulkCampaignAgeTargetBid

        :return: the wrapping multi-line entity
        :rtype: :class:`._MultiRecordBulkEntity`
        """
        raise NotImplementedError()

    def convert_to_values(self, row_values, mappings):
        for mapping in mappings:
            try:
                mapping.convert_to_csv(self, row_values)
            except Exception as ex:
                raise self._create_entity_write_error(mapping, ex)

    def _create_entity_write_error(self, mapping, ex):
        entity_type = str(type(self))
        if isinstance(mapping, _SimpleBulkMapping):
            message = "Couldn't write column {0} of {1} entity: {2}".format(mapping.header, entity_type, ex)
        else:
            message = "Couldn't write {0} entity: {1}".format(entity_type, ex)
        message += " See InnerException for error details."
        return EntityWriteException(message=message, inner_exception=ex)



# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from typing_extensions import Self

class SuggestedResponsiveSearchAd(BaseModel):
    """
    SuggestedResponsiveSearchAd
    """ # noqa: E501
    headlines: Optional[List[StrictStr]] = Field(default=None, alias="Headlines")
    descriptions: Optional[List[StrictStr]] = Field(default=None, alias="Descriptions")
    path1: Optional[StrictStr] = Field(default=None, alias="Path1")
    path2: Optional[StrictStr] = Field(default=None, alias="Path2")
    final_url: Optional[StrictStr] = Field(default=None, alias="FinalUrl")
    __properties: ClassVar[List[str]] = ["Headlines", "Descriptions", "Path1", "Path2", "FinalUrl"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if headlines (nullable) is None
        # and model_fields_set contains the field
        if self.headlines is None and "headlines" in self.model_fields_set:
            _dict['Headlines'] = None

        # set to None if descriptions (nullable) is None
        # and model_fields_set contains the field
        if self.descriptions is None and "descriptions" in self.model_fields_set:
            _dict['Descriptions'] = None

        # set to None if path1 (nullable) is None
        # and model_fields_set contains the field
        if self.path1 is None and "path1" in self.model_fields_set:
            _dict['Path1'] = None

        # set to None if path2 (nullable) is None
        # and model_fields_set contains the field
        if self.path2 is None and "path2" in self.model_fields_set:
            _dict['Path2'] = None

        # set to None if final_url (nullable) is None
        # and model_fields_set contains the field
        if self.final_url is None and "final_url" in self.model_fields_set:
            _dict['FinalUrl'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SuggestedResponsiveSearchAd from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Headlines": obj.get("Headlines"),
                        "Descriptions": obj.get("Descriptions"),
                        "Path1": obj.get("Path1") if obj.get("Path1") is not None else None,
                        "Path2": obj.get("Path2") if obj.get("Path2") is not None else None,
                        "FinalUrl": obj.get("FinalUrl") if obj.get("FinalUrl") is not None else None
        })
        return _obj



# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from typing_extensions import Self

class AuctionInsightKpi(BaseModel):
    """
    AuctionInsightKpi
    """ # noqa: E501
    segments: Optional[List[StrictStr]] = Field(default=None, alias="Segments")
    impression_share: Optional[StrictFloat] = Field(default=None, alias="ImpressionShare")
    overlap_rate: Optional[StrictFloat] = Field(default=None, alias="OverlapRate")
    average_position: Optional[StrictFloat] = Field(default=None, alias="AveragePosition")
    above_rate: Optional[StrictFloat] = Field(default=None, alias="AboveRate")
    top_of_page_rate: Optional[StrictFloat] = Field(default=None, alias="TopOfPageRate")
    outranking_share: Optional[StrictFloat] = Field(default=None, alias="OutrankingShare")
    absolute_top_of_page_rate: Optional[StrictFloat] = Field(default=None, alias="AbsoluteTopOfPageRate")
    __properties: ClassVar[List[str]] = ["Segments", "ImpressionShare", "OverlapRate", "AveragePosition", "AboveRate", "TopOfPageRate", "OutrankingShare", "AbsoluteTopOfPageRate"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if segments (nullable) is None
        # and model_fields_set contains the field
        if self.segments is None and "segments" in self.model_fields_set:
            _dict['Segments'] = None

        # set to None if impression_share (nullable) is None
        # and model_fields_set contains the field
        if self.impression_share is None and "impression_share" in self.model_fields_set:
            _dict['ImpressionShare'] = None

        # set to None if overlap_rate (nullable) is None
        # and model_fields_set contains the field
        if self.overlap_rate is None and "overlap_rate" in self.model_fields_set:
            _dict['OverlapRate'] = None

        # set to None if average_position (nullable) is None
        # and model_fields_set contains the field
        if self.average_position is None and "average_position" in self.model_fields_set:
            _dict['AveragePosition'] = None

        # set to None if above_rate (nullable) is None
        # and model_fields_set contains the field
        if self.above_rate is None and "above_rate" in self.model_fields_set:
            _dict['AboveRate'] = None

        # set to None if top_of_page_rate (nullable) is None
        # and model_fields_set contains the field
        if self.top_of_page_rate is None and "top_of_page_rate" in self.model_fields_set:
            _dict['TopOfPageRate'] = None

        # set to None if outranking_share (nullable) is None
        # and model_fields_set contains the field
        if self.outranking_share is None and "outranking_share" in self.model_fields_set:
            _dict['OutrankingShare'] = None

        # set to None if absolute_top_of_page_rate (nullable) is None
        # and model_fields_set contains the field
        if self.absolute_top_of_page_rate is None and "absolute_top_of_page_rate" in self.model_fields_set:
            _dict['AbsoluteTopOfPageRate'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AuctionInsightKpi from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Segments": obj.get("Segments"),
                        "ImpressionShare": obj.get("ImpressionShare") if obj.get("ImpressionShare") is not None else None,
                        "OverlapRate": obj.get("OverlapRate") if obj.get("OverlapRate") is not None else None,
                        "AveragePosition": obj.get("AveragePosition") if obj.get("AveragePosition") is not None else None,
                        "AboveRate": obj.get("AboveRate") if obj.get("AboveRate") is not None else None,
                        "TopOfPageRate": obj.get("TopOfPageRate") if obj.get("TopOfPageRate") is not None else None,
                        "OutrankingShare": obj.get("OutrankingShare") if obj.get("OutrankingShare") is not None else None,
                        "AbsoluteTopOfPageRate": obj.get("AbsoluteTopOfPageRate") if obj.get("AbsoluteTopOfPageRate") is not None else None
        })
        return _obj
